@@ -10,7 +10,7 @@
  *		Internet: MRC@CAC.Washington.EDU
  *
  * Date:	25 January 1993
- * Last Edited:	21 August 1994
+ * Last Edited:	9 October 1994
  *
  * Copyright 1994 by the University of Washington
  *
@@ -49,6 +49,7 @@
 #include "nntpcdos.h"
 #include "rfc822.h"
 #include "misc.h"
+#include "newsrc.h"
 
 /* NNTP mail routines */
 
@@ -93,9 +94,6 @@ DRIVER nntpdriver = {
 
 				/* prototype stream */
 MAILSTREAM nntpproto = {&nntpdriver};
-
-				/* driver parameters */
-static char *nntp_newsrc = NIL;	/* news directory name */
 
 /* NNTP mail validate mailbox
  * Accepts: mailbox name
@@ -117,24 +115,7 @@ DRIVER *nntp_valid (char *name)
 
 void *nntp_parameters (long function,void *value)
 {
-  switch ((int) function) {
-  case SET_NEWSRC:
-    if (nntp_newsrc) fs_give ((void **) &nntp_newsrc);
-    nntp_newsrc = cpystr ((char *) value);
-    break;
-  case GET_NEWSRC:
-    if (!nntp_newsrc) {		/* set news directory name if not defined */
-      char tmp[MAILTMPLEN];
-      sprintf (tmp,"%s\\NEWSRC",myhomedir ());
-      nntp_newsrc = cpystr (tmp);
-    }
-    value = (void *) nntp_newsrc;
-    break;
-  default:
-    value = NIL;		/* error case */
-    break;
-  }
-  return value;
+  return NIL;
 }
 
 /* NNTP mail find list of mailboxes
@@ -155,35 +136,10 @@ void nntp_find (MAILSTREAM *stream,char *pat)
 
 void nntp_find_bboards (MAILSTREAM *stream,char *pat)
 {
-  FILE *f = NIL;
-  void *s;
-  char *t,*u,*bbd,*patx,tmp[MAILTMPLEN],patc[MAILTMPLEN];
-  if (stream) {			/* use .newsrc if a stream given */
-				/* begin with a host specification? */
-    if (((*pat == '{') || ((*pat == '*') && (pat[1] == '{'))) &&
-	(t = strchr (pat,'}')) && *(patx = ++t)) {
-      if (*pat == '*') pat++;	/* yes, skip leading * (old Pine behavior) */
-      strcpy (tmp,pat);		/* copy host name */
-      bbd = tmp + (patx - pat);	/* where we write the bboards */
-    }
-    else {			/* no host specification */
-      bbd = tmp;		/* no prefix */
-      patx = pat;		/* use entire specification */
-    }
-				/* check all newsgroups from .newsrc */
-    while (t = nntp_read_sdb (&f)) {
-      if (u = strchr (t,':')) {	/* subscribed newsgroup? */
-	*u = '\0';		/* tie off at end of name */
-	if (pmatch (t,patx)) {	/* pattern patch */
-	  strcpy (bbd,t);	/* write newsgroup name after hostspec */
-	  mm_bboard (tmp);
-	}
-      }
-      fs_give ((void **) &t);	/* discard the line */
-    }
-  }
-				/* use subscription manager if no stream */
-  else while (t = sm_read (&s))
+  char *t;
+  void *s = NIL;
+  if (stream) newsrc_find (pat);/* use .newsrc if a stream given */
+  else while (t = sm_read (&s))	/* use subscription manager if no stream */
     if ((*t == '*') && pmatch (t+1,pat)) mm_bboard (t+1);
 }
 
@@ -267,7 +223,7 @@ long nntp_unsubscribe (MAILSTREAM *stream,char *mailbox)
 long nntp_subscribe_bboard (MAILSTREAM *stream,char *mailbox)
 {
   char *s = strchr (mailbox,'}');
-  return s ? nntp_update_sdb (s+1,":") : NIL;
+  return s ? newsrc_update (s+1,':') : NIL;
 }
 
 
@@ -280,7 +236,7 @@ long nntp_subscribe_bboard (MAILSTREAM *stream,char *mailbox)
 long nntp_unsubscribe_bboard (MAILSTREAM *stream,char *mailbox)
 {
   char *s = strchr (mailbox,'}');
-  return s ? nntp_update_sdb (s+1,"!") : NIL;
+  return s ? newsrc_update (s+1,'!') : NIL;
 }
 
 /* NNTP mail create mailbox
@@ -327,12 +283,10 @@ MAILSTREAM *nntp_mopen (MAILSTREAM *stream)
 {
   long i,j,k;
   long nmsgs = 0;
-  long recent = 0;
   long unseen = 0;
   static int tmpcnt = 0;
   char c = NIL,*s,*t,tmp[MAILTMPLEN];
   NETMBX mb;
-  FILE *f = NIL;
   void *tcpstream;
   SMTPSTREAM *nstream = NIL;
 				/* return prototype for OP_PROTOTYPE call */
@@ -387,9 +341,8 @@ MAILSTREAM *nntp_mopen (MAILSTREAM *stream)
     sprintf (tmp,"%s\\NEWS%4.4d.TMP",s,++tmpcnt);
     LOCAL->tmp = cpystr (tmp);	/* build name of tmp file we use */
     stream->sequence++;		/* bump sequence number */
-    stream->readonly = T;	/* make sure higher level knows readonly */
+    stream->rdonly = T;		/* make sure higher level knows readonly */
     LOCAL->number = NIL;
-    LOCAL->seen = NIL;
 
     if (!stream->halfopen) {	/* if not half-open */
       if (nmsgs) {		/* find what messages exist */
@@ -410,61 +363,10 @@ MAILSTREAM *nntp_mopen (MAILSTREAM *stream)
 	}
 				/* assume c-client/NNTP map is entire range */
 	else for (k = 0; k < nmsgs; ++k) LOCAL->number[k] = i + k;
-				/* create state flag cache */
-	LOCAL->seen = (char *) fs_get (nmsgs * sizeof (char));
-				/* initialize state */
-	for (i = 0; i < nmsgs; ++i) LOCAL->seen[i] = NIL;
       }
-
       mail_exists(stream,nmsgs);/* notify upper level about mailbox size */
-      i = 0;			/* nothing scanned yet */
-      s = NIL;
-      while (t = nntp_read_sdb (&f)) {
-	if ((s = strpbrk (t,":!")) && (c = *s)) {
-	  *s++ = '\0';		/* tie off newsgroup name, point to data */
-	  if (strcmp (t,LOCAL->name)) s = NIL;
-	  else break;		/* found it! */
-	}
-	fs_give ((void **) &t);	/* give back this entry line */
-      }
-      if (s) {			/* newsgroup found? */
-	if (*s == ' ') s++;	/* skip whitespace */
-	while (*s && i < nmsgs){/* process until run out of messages or list */
-	  j = strtol (s,&s,10);	/* start of possible range */
-				/* other end of range */
-	  k = (*s == '-') ? strtol (++s,&s,10) : j;
-				/* skip messages before this range */
-	  while ((LOCAL->number[i] < j) && (i < nmsgs)) {
-	    if (!unseen) unseen = i + 1;
-	    i++;
-	  }
-	  while ((LOCAL->number[i] >= j) && (LOCAL->number[i] <= k) &&
-		 (i < nmsgs)){	/* mark messages within the range as seen */
-	    LOCAL->seen[i++] = T;
-	    mail_elt (stream,i)->deleted = T;
-	  }
-	  if (*s == ',') s++;	/* skip past comma */
-	  else if (*s) {	/* better not be anything else then */
-	    mm_log ("Bogus syntax in news state file",ERROR);
-	    break;		/* give up fast!! */
-	  }
-	}
-      }
-      else mm_log ("No state for newsgroup found, reading as new",WARN);
-      if (t) {			/* need to free up cruft? */
-	fs_give ((void **) &t);	/* yes, give back newsrc entry */
-	fclose (f);		/* close the file */
-      }
-      while (i++ < nmsgs) {	/* mark all remaining messages as new */
-	mail_elt (stream,i)->recent = T;
-	++recent;		/* count another recent message */
-      }
-      if (unseen) {		/* report first unseen message */
-	sprintf (tmp,"[UNSEEN] %ld is first unseen message",unseen);
-	mm_notify (stream,tmp,(long) NIL);
-      }
-				/* notify upper level about recent */
-      mail_recent(stream,recent);
+				/* read .newsrc entries */
+      mail_recent (stream,newsrc_read (LOCAL->name,stream,LOCAL->number));
 				/* notify if empty bboard */
       if (!(stream->nmsgs||stream->silent)) mm_log ("Newsgroup is empty",WARN);
     }
@@ -487,7 +389,6 @@ void nntp_close (MAILSTREAM *stream)
     if (LOCAL->name) fs_give ((void **) &LOCAL->name);
     if (LOCAL->host) fs_give ((void **) &LOCAL->host);
     if (LOCAL->number) fs_give ((void **) &LOCAL->number);
-    if (LOCAL->seen) fs_give ((void **) &LOCAL->seen);
 				/* close NNTP connection */
     if (LOCAL->nntpstream) smtp_close (LOCAL->nntpstream);
 				/* nuke the local data */
@@ -503,7 +404,13 @@ void nntp_close (MAILSTREAM *stream)
 
 void nntp_fetchfast (MAILSTREAM *stream,char *sequence)
 {
-  return;			/* no-op for local mail */
+  long i;
+  BODY *b;
+				/* ugly and slow */
+  if (stream && LOCAL && mail_sequence (stream,sequence))
+    for (i = 1; i <= stream->nmsgs; i++)
+      if (mail_elt (stream,i)->sequence)
+	nntp_fetchstructure (stream,i,&b);
 }
 
 
@@ -570,7 +477,6 @@ ENVELOPE *nntp_fetchstructure (MAILSTREAM *stream,long msgno,BODY **body)
       close (LOCAL->fd);	/* clean up the file descriptor */
     }
     else {			/* failed, mark as deleted */
-      LOCAL->seen[m] = T;
       mail_elt (stream,msgno)->deleted = T;
       h = cpystr ("");		/* no header */
       hdrsize = 0;		/* empty header */
@@ -582,15 +488,14 @@ ENVELOPE *nntp_fetchstructure (MAILSTREAM *stream,long msgno,BODY **body)
 	d.fd = LOCAL->fd;	/* set initial stringstruct */
 	d.pos = 0;		/* starting at beginning of file */
 	INIT (&bs,dawz_string,(void *) &d,textsize);
+				/* calculate message size */
+	elt->rfc822_size = hdrsize + textsize;
       }
       else {			/* failed, mark as deleted */
-	LOCAL->seen[m] = T;
 	mail_elt (stream,msgno)->deleted = T;
 	body = NIL;		/* failed or some reason */
       }
     }
-				/* calculate message size */
-    elt->rfc822_size = hdrsize + textsize;
 				/* parse envelope and body */
     rfc822_parse_msg (env,body ? b : NIL,h,hdrsize,body ? &bs:NIL,BADHOST,tmp);
     fs_give ((void **) &h);	/* don't need header any more */
@@ -615,21 +520,18 @@ char *nntp_fetchheader (MAILSTREAM *stream,long msgno)
   char tmp[MAILTMPLEN];
   unsigned long hdrsize;
   long m = msgno - 1;
-				/* set default gets routine */
-  if (!mailgets) mailgets = mm_gets;
+  mailgets_t mg = (mailgets_t) mail_parameters (NIL,GET_GETS,NIL);
   if (stream->text) fs_give ((void **) &stream->text);
 				/* get the header */
   sprintf (tmp,"%ld",LOCAL->number[m]);
   if (smtp_send (LOCAL->nntpstream,"HEAD",tmp) == NNTPHEAD) {
     nntp_slurp (stream,&hdrsize);
 				/* return it to user */
-    stream->text = (*mailgets) (nntp_read,stream,hdrsize);
+    stream->text = (mg ? *mg : mm_gets) (nntp_read,stream,hdrsize);
     close (LOCAL->fd);		/* clean up the file descriptor */
   }
-  else {			/* failed, mark as deleted */
-    LOCAL->seen[m] = T;
-    mail_elt (stream,msgno)->deleted = T;
-  }
+				/* failed, mark as deleted */
+  else mail_elt (stream,msgno)->deleted = T;
   return stream->text ? stream->text : "";
 }
 
@@ -646,21 +548,17 @@ char *nntp_fetchtext (MAILSTREAM *stream,long msgno)
   unsigned long textsize;
   long m = msgno - 1;
   char tmp[MAILTMPLEN];
-				/* set default gets routine */
-  if (!mailgets) mailgets = mm_gets;
+  mailgets_t mg = (mailgets_t) mail_parameters (NIL,GET_GETS,NIL);
   if (stream->text) fs_give ((void **) &stream->text);
 				/* get the text */
   sprintf (tmp,"%ld",LOCAL->number[m]);
   if (smtp_send (LOCAL->nntpstream,"BODY",tmp) == NNTPBODY) {
     nntp_slurp (stream,&textsize);
 				/* return it to user */
-    stream->text = (*mailgets) (nntp_read,stream,textsize);
+    stream->text = (mg ? *mg : mm_gets) (nntp_read,stream,textsize);
     close (LOCAL->fd);		/* clean up the file descriptor */
   }
-  else {			/* failed, mark as deleted */
-    LOCAL->seen[m] = T;
-    mail_elt (stream,msgno)->deleted = T;
-  }
+  else elt->deleted = T;	/* failed, mark as deleted */
   elt->seen = T;		/* mark as seen */
   return stream->text ? stream->text : "";
 }
@@ -682,8 +580,7 @@ char *nntp_fetchbody (MAILSTREAM *stream,long m,char *s,unsigned long *len)
   unsigned long textsize;
   MESSAGECACHE *elt = mail_elt (stream,m);
   char tmp[MAILTMPLEN];
-				/* set default gets routine */
-  if (!mailgets) mailgets = mm_gets;
+  mailgets_t mg = (mailgets_t) mail_parameters (NIL,GET_GETS,NIL);
   if (stream->text) fs_give ((void **) &stream->text);
 				/* make sure have a body */
   if (!(nntp_fetchstructure (stream,m,&b) && b && s && *s &&
@@ -719,14 +616,11 @@ char *nntp_fetchbody (MAILSTREAM *stream,long m,char *s,unsigned long *len)
     nntp_slurp (stream,&textsize);
 				/* move to that place in the file */
     lseek (LOCAL->fd,offset,SEEK_SET);
-				/* return it to user */
-    stream->text = (*mailgets) (nntp_read,stream,*len = b->size.ibytes);
+    stream->text =		/* return it to user */
+      (mg ? *mg : mm_gets) (nntp_read,stream,*len = b->size.ibytes);
     close (LOCAL->fd);		/* clean up the file descriptor */
   }
-  else {			/* failed, mark as deleted */
-    mail_elt (stream,m)->deleted = T;
-    LOCAL->seen[m - 1] = T;
-  }
+  else elt->deleted = T;	/* failed, mark as deleted */
   elt->seen = T;		/* mark as seen */
   return stream->text ? stream->text : "";
 }
@@ -811,7 +705,7 @@ void nntp_setflag (MAILSTREAM *stream,char *sequence,char *flag)
       if (f&fSEEN) elt->seen=T;	/* set all requested flags */
       if (f&fDELETED) {		/* deletion marks it in newsrc */
 	elt->deleted = T;	/* mark deleted */
-	if (!LOCAL->seen[i]) LOCAL->seen[i] = LOCAL->dirty = T;
+	LOCAL->dirty = T;	/* mark dirty */
       }
       if (f&fFLAGGED) elt->flagged = T;
       if (f&fANSWERED) elt->answered = T;
@@ -838,10 +732,7 @@ void nntp_clearflag (MAILSTREAM *stream,char *sequence,char *flag)
       if (f&fSEEN) elt->seen = NIL;
       if (f&fDELETED) {
 	elt->deleted = NIL;	/* undelete */
-	if (LOCAL->seen[i]) {	/* if marked in newsrc */
-	  LOCAL->seen[i] = NIL;	/* unmark it now */
-	  LOCAL->dirty = T;	/* mark stream as dirty */
-	}
+	LOCAL->dirty = T;	/* mark stream as dirty */
       }
       if (f&fFLAGGED) elt->flagged = NIL;
       if (f&fANSWERED) elt->answered = NIL;
@@ -968,32 +859,9 @@ long nntp_ping (MAILSTREAM *stream)
 
 void nntp_check (MAILSTREAM *stream)
 {
-  unsigned long i,j,k;
-  char *s,tmp[MAILTMPLEN];
-  if (!LOCAL->dirty) return;	/* never do if no updates */
-  *(s = tmp) = '\0';		/* initialize list */
-  for (i = 0,j = 1,k = 0; i < stream->nmsgs; ++i) {
-    if (LOCAL->seen[i]) {	/* seen message? */
-      k = LOCAL->number[i];	/* this is the top of the current range */
-      if (j == 0) j = k;	/* if no range in progress, start one */
-    }
-    else if (j != 0) {		/* unread message, ending a range */
-				/* calculate end of range */
-      if (k = LOCAL->number[i] - 1) {
-				/* dump range */
-	sprintf (s,(j == k) ? "%ld," : "%ld-%ld,",j,k);
-	s += strlen (s);	/* find end of string */
-      }
-      j = 0;			/* no more range in progress */
-    }
-  }
-  if (j) {			/* dump trailing range */
-    sprintf (s,(j == k) ? "%ld" : "%ld-%ld",j,k);
-    s += strlen (s);		/* find end of string */
-  }
-  else if (s[-1] == ',') s--;	/* prepare to patch out any trailing comma */
-  *s++ = '\0';			/* tie off string */
-  nntp_update_sdb (LOCAL->name,tmp);
+				/* never do if no updates */
+  if (LOCAL->dirty) newsrc_write (LOCAL->name,stream,LOCAL->number);
+  LOCAL->dirty = NIL;
 }
 
 /* NNTP mail expunge mailbox
@@ -1062,121 +930,6 @@ void nntp_gc (MAILSTREAM *stream,long gcflags)
 /* Internal routines */
 
 
-/* Read NNTP database
- * Accepts: pointer to subscription database file handle (NIL if first time)
- * Returns: line from the file
- */
-
-char *nntp_read_sdb (FILE **f)
-{
-  int i;
-  char *s,*t,tmp[MAILTMPLEN];
-				/* if first time, open newsrc file */
-  if (!(*f || (*f = fopen (NEWSRC (tmp),"r")))) {
-    char msg[MAILTMPLEN];
-    sprintf (msg,"No news state found, will create %s",tmp);
-    mm_log (msg,WARN);
-    if ((i = open (tmp,O_WRONLY|O_APPEND|O_CREAT,S_IREAD|S_IWRITE)) >= 0)
-      close (i);
-    return NIL;
-  }
-				/* read a line from the file */
-  if (fgets (tmp,MAILTMPLEN,*f)) {
-    i = strlen (tmp);		/* how many characters we got */
-    if (tmp[i - 1] == '\n') {	/* got a complete line? */
-      tmp[i - 1] = '\0';	/* yes, tie off the line */
-      s = cpystr (tmp);		/* make return string */
-    }
-    else {			/* ugh, have to build from fragments */
-      t = nntp_read_sdb (f);	/* recuse to get remainder */
-      sprintf (s = (char *) fs_get (i + strlen (t) + 1),"%s%s",tmp,t);
-      fs_give ((void **) &t);	/* discard fragment */
-    }
-    return s;			/* return the string we made */
-  }
-  fclose (*f);			/* end of file, close file */
-  *f = NIL;			/* make sure caller knows */
-  return NIL;			/* all done */
-}
-
-/* Update NNTP database
- * Accepts: newsgroup name
- *	    message data
- * Returns: T if success, NIL if failure
- */
-
-long nntp_update_sdb (char *name,char *data)
-{
-  int i = strlen (name);
-  char c,*s,tmp[MAILTMPLEN],new[MAILTMPLEN];
-  FILE *f = NIL,*of,*nf;
-				/* make name of old newsrc */
-  if ((s = strrchr (NEWSRC (tmp),'\\')) && (s = strrchr (tmp,'.')))
-    strcpy (s,".OLD");
-  else strcat (tmp,".OLD");
-  if (!(of = fopen (tmp,"w"))) {/* open old newsrc */
-    mm_log ("Can't create backup of news state",ERROR);
-    return NIL;
-  }
-				/* make name of new newsrc */
-  if ((s = strrchr (NEWSRC (new),'\\')) && (s = strrchr (new,'.')))
-    strcpy (s,".NEW");
-  else strcat (new,".NEW");
-  if (!(nf = fopen (new,"w"))) {/* open new newsrc */
-    mm_log ("Can't create new news state",ERROR);
-    fclose (of);
-    return NIL;
-  }
-				/* process .newsrc file */
-  while (s = nntp_read_sdb (&f)) {
-    fprintf (of,"%s\n",s);	/* write to backup file */
-    if (data && (!strncmp (s,name,i)) && (((c = s[i]) == ':') || (c == '!'))) {
-      switch (*data) {		/* first time we saw this entry... */
-      case ':':			/* subscription request */
-	if (c == '!') c = ':';	/* subscribe if unsubscribed */
-	else {			/* complain if already subscribed */
-	  sprintf (tmp,"Already subscribed to newsgroup %s",name);
-	  mm_log (tmp,WARN);
-	}
-	data = s + i + 1;	/* preserve old read state */
-	break;
-      case '!':			/* unsubscription request? */
-	if (c == ':') c = '!';	/* unsubscribe if subscribed */
-	data = s + i + 1;	/* preserve old read state */
-	break;
-      default:			/* update read state */
-	break;
-      }
-				/* write the new entry */
-      fprintf (nf,"%s%c%s\n",name,c,data);
-      data = NIL;		/* request satisfied */
-    }
-    else fprintf (nf,"%s\n",s);	/* not the entry we want, write to new file */
-    fs_give ((void **) &s);
-  }
-
-  if (data) switch (*data) {	/* if didn't find it, make new entry */
-  case ':':			/* subscription request */
-    fprintf (nf,"%s: \n",name);
-    break;
-  case '!':			/* unsubscription request */
-    fprintf (nf,"%s! \n",name);
-    break;
-  default:			/* update read state */
-    fprintf (nf,"%s: %s\n",name,data);
-    break;
-  }
-  fclose (nf);			/* close new file */
-  fclose (of);			/* close backup file */
-  unlink (NEWSRC (tmp));	/* remove the current file */
-  if (rename (new,tmp)) {	/* rename new database to current */
-    mm_log ("Can't update news state",ERROR);
-    return NIL;
-  }
-  return LONGT;			/* return success */
-}	 
-
-
 /* Parse flag list
  * Accepts: MAIL stream
  *	    flag list as a character string
@@ -1491,28 +1244,30 @@ search_t nntp_search_flag (search_t f,char **d)
 
 search_t nntp_search_string (search_t f,char **d,long *n)
 {
+  char *end = " ";
   char *c = strtok (NIL,"");	/* remainder of criteria */
-  if (c) {			/* better be an argument */
-    switch (*c) {		/* see what the argument is */
-    case '\0':			/* catch bogons */
-    case ' ':
-      return NIL;
-    case '"':			/* quoted string */
-      if (!(strchr (c+1,'"') && (*d = strtok (c,"\"")) && (*n = strlen (*d))))
-	return NIL;
-      break;
-    case '{':			/* literal string */
-      *n = strtol (c+1,&c,10);	/* get its length */
-      if (*c++ != '}' || *c++ != '\015' || *c++ != '\012' ||
-	  *n > strlen (*d = c)) return NIL;
-      c[*n] = DELIM;		/* write new delimiter */
-      strtok (c,DELMS);		/* reset the strtok mechanism */
-      break;
-    default:			/* atomic string */
-      *n = strlen (*d = strtok (c," "));
+  if (!c) return NIL;		/* missing argument */
+  switch (*c) {			/* see what the argument is */
+  case '{':			/* literal string */
+    *n = strtol (c+1,d,10);	/* get its length */
+    if ((*(*d)++ == '}') && (*(*d)++ == '\015') && (*(*d)++ == '\012') &&
+	(!(*(c = *d + *n)) || (*c == ' '))) {
+      char e = *--c;
+      *c = DELIM;		/* make sure not a space */
+      strtok (c," ");		/* reset the strtok mechanism */
+      *c = e;			/* put character back */
       break;
     }
-    return f;
+  case '\0':			/* catch bogons */
+  case ' ':
+    return NIL;
+  case '"':			/* quoted string */
+    if (strchr (c+1,'"')) end = "\"";
+    else return NIL;
+  default:			/* atomic string */
+    if (*d = strtok (c,end)) *n = strlen (*d);
+    else return NIL;
+    break;
   }
-  else return NIL;
+  return f;
 }
