@@ -7,7 +7,7 @@
  *		Internet: MRC@Panda.COM
  *
  * Date:	26 January 1992
- * Last Edited:	26 June 1994
+ * Last Edited:	14 September 1994
  *
  * Copyright 1994 by Mark Crispin
  *
@@ -49,9 +49,8 @@
 #include "osdep.h"
 #include "misc.h"
 
-short TCPdriver = 0;		/* MacTCP's reference number */
+static short TCPdriver = 0;	/* MacTCP's reference number */
 short resolveropen = 0;		/* TCP's resolver open */
-static char *myLocalHost = NIL;	/* local host name */
 
 
 #include "env_mac.c"
@@ -59,25 +58,3 @@ static char *myLocalHost = NIL;	/* local host name */
 #include "ftl_mac.c"
 #include "nl_mac.c"
 #include "tcp_mac.c"
-
-/* This is here instead of env_mac.c so the Mint port doesn't get confused */
-
-/* Determine default prototype stream to user
- * Returns: default prototype stream
- */
-
-MAILSTREAM *default_proto ()
-{
-  extern MAILSTREAM dummyproto;
-  return &dummyproto;		/* return default driver's prototype */
-}
-
-
-/* Return my local host name
- * Returns: my local host name
- */
-
-char *mylocalhost (void)
-{
-  return myLocalHost ? myLocalHost : "random-mac";
-}
