@@ -10,7 +10,7 @@
  *		Internet: MRC@CAC.Washington.EDU
  *
  * Date:	8 July 1988
- * Last Edited:	2 August 1994
+ * Last Edited:	4 September 1994
  *
  * Sponsorship:	The original version of this work was developed in the
  *		Symbolic Systems Resources Group of the Knowledge Systems
@@ -110,37 +110,37 @@ int main ()
   MAILSTREAM *stream = NIL;
   char tmp[MAILTMPLEN];
   long debug;
+#include "linkage.c"
 #if MACOS
-  size_t *base = (size_t *) 0x000908;
+  {
+    size_t *base = (size_t *) 0x000908;
 				/* increase stack size on a Mac */
-  SetApplLimit ((Ptr) (*base - (size_t) 65535L));
+    SetApplLimit ((Ptr) (*base - (size_t) 65535L));
+  }
 #endif
 #if UNIXLIKE
-  char *name;
-  char *suffix;
-  struct passwd *pwd;
-  struct hostent *host_name;
 #ifdef __MINT__
-  mint_setup ();
+    mint_setup ();
 #endif
-				/* get user name and passwd entry */
-  if (name = (char *) getlogin ()) pwd = getpwnam (name);
-  else {
-    pwd = getpwuid (getuid ());	/* get it this way if detached, etc */
-    name = pwd->pw_name;
-  }
-  curusr = cpystr (name);	/* current user is this name */
-  strcpy (tmp,pwd->pw_gecos);	/* probably not necessay but be safe */
+  curusr = cpystr(myusername());/* current user is this name */
+  {
+    char *suffix;
+    struct passwd *pwd = getpwnam (curusr);
+    if (pwd) {
+      strcpy (tmp,pwd->pw_gecos);
 				/* dyke out the office and phone poop */
-  if (suffix = strchr (tmp,',')) suffix[0] = '\0';
-  strcpy (personalname,tmp);	/* make a permanent copy of it */
+      if (suffix = strchr (tmp,',')) suffix[0] = '\0';
+      strcpy (personalname,tmp);/* make a permanent copy of it */
+    }
+    else personalname[0] = '\0';
+  }
 #else
-  prompt ("Personal name: ",personalname);
   curusr = cpystr ("somebody");
+  personalname[0] = '\0';
 #endif
   curhst = cpystr (mylocalhost ());
   puts ("MTest -- C client test program");
-#include "linkage.c"
+  if (!*personalname) prompt ("Personal name: ",personalname);
 				/* user wants protocol telemetry? */
   prompt ("Debug protocol (y/n)?",tmp);
   ucase (tmp);
