@@ -1,5 +1,5 @@
 #if	!defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: attach.c,v 4.16 1994/08/03 03:04:02 mikes Exp $";
+static char rcsid[] = "$Id: attach.c,v 4.18 1994/09/29 19:15:54 mikes Exp $";
 #endif
 /*
  * Program:	Routines to support attachments in the Pine composer 
@@ -125,7 +125,7 @@ char *fn, *sz, *cmnt;
 	    if(*fn == '\0' || !isdir(fn, NULL))
 	      strcpy(fn, gethomedir(NULL));
 
-	    if(FileBrowse(fn, bfn, sz) == 1){
+	    if(FileBrowse(fn, bfn, sz, FB_READ) == 1){
 		strcat(fn, S_FILESEP);
 		strcat(fn, bfn);
 		i--;
@@ -425,10 +425,14 @@ int  *no;					/* attachment number        */
 		p = tmp;
 		level = WSN;			/* what's next... */
 	    }
-	    else if(c < '0' || c > '9'){
-		*p = '\0';
-		emlwrite("\007Attchmnt: Number field missing '.': \"%s\"",tmp);
-		rv = -1;
+	    else if(c < '0' || c > '9'){	/* Must be part of tag */
+		sprintf(number, "%d. ", *no);
+		sinserts((*lp == NULL) ? &lprev->text[(*off) - (p - tmp)]
+			               : &(*lp)->text[(*off) - (p - tmp)],
+			     0, number, j=strlen(number));
+		*off += j;
+		*p++ = c;
+		level = TAG;			/* interpret the name */
 	    }
 	    else
 	      *p++ = c;
