@@ -1,5 +1,5 @@
 #if	!defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: display.c,v 4.26 1994/08/09 18:17:28 mikes Exp $";
+static char rcsid[] = "$Id: display.c,v 4.27 1994/10/10 21:56:33 mikes Exp $";
 #endif
 /*
  * Program:	Display functions
@@ -935,7 +935,7 @@ WINDOW *wp;
 
 	n = 0;
 	c.a = 1;
-#if     MSDOS
+#if	defined(MSDOS) || defined(LINUX)
 	vtmove(1, 0);
 	vteeol();
 #endif
