@@ -1,5 +1,5 @@
 /*
- * $Id: efunc.h,v 4.11 1994/06/16 02:09:24 mikes Exp $
+ * $Id: efunc.h,v 4.12 1994/09/27 16:04:00 mikes Exp $
  *
  * Program:	Pine's composer and pico's function declarations
  *
@@ -89,7 +89,7 @@ extern	int normal(int, int (*)[2],int);
 extern	int rebindfunc(int (*a)(int, int),int (*)(int, int));
 
 /* browse.c */
-extern	int FileBrowse(char *, char *, char *);
+extern	int FileBrowse(char *, char *, char *, int);
 extern	int ResizeBrowser(void);
 
 /* buffer.c */
