@@ -1,5 +1,5 @@
 #if	!defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: os_unix.c,v 4.49 1994/08/11 19:13:27 mikes Exp $";
+static char rcsid[] = "$Id: os_unix.c,v 4.51 1994/10/10 23:28:56 mikes Exp $";
 #endif
 /*
  * Program:	Operating system dependent routines - Ultrix 4.1
@@ -344,10 +344,10 @@ ttgetc()
     int i;
 
     if((i = read(0, &c, 1)) <= 0){
-	if(i == 0 || errno == EINTR)
+	if(i == 0 && errno == EINTR)		/* only acceptable failure */
 	  return(NODATA);
 	else
-	  kill(getpid(), SIGHUP);
+	  kill(getpid(), SIGHUP);		/* fake a hup */
     }
     else
       return((int)c);
@@ -527,7 +527,7 @@ alt_editor(f, n)
     char   *writetmp();
     int	   child, pid, i, done = 0;
     long   l;
-#if	defined(POSIX) || defined(sv3) || defined(COHERENT) || defined(isc)
+#if	defined(POSIX) || defined(sv3) || defined(COHERENT) || defined(isc) || defined(neb)
     int    stat;
 #else
     union  wait stat;
@@ -951,7 +951,9 @@ char *
 errstr(err)
 int err;
 {
+#ifndef	neb
     extern char *sys_errlist[];
+#endif
     extern int  sys_nerr;
 
     return((err >= 0 && err < sys_nerr) ? sys_errlist[err] : NULL);
