@@ -1,5 +1,5 @@
 /*
- * $Id: pico.h,v 4.22 1994/07/15 23:33:28 mikes Exp $
+ * $Id: pico.h,v 4.24 1994/09/27 16:21:49 mikes Exp $
  *
  * Program:	pico.h - definitions for Pine's composer library
  *
@@ -213,13 +213,20 @@ typedef struct pico_struct {
  */
 #ifdef	maindef
 PICO	*Pmaster = NULL;		/* composer specific stuff */
-char	*version = "2.4";		/* PICO version number */
+char	*version = "2.5";		/* PICO version number */
 #else
 extern PICO *Pmaster;			/* composer specific stuff */
 extern char *version;			/* pico version! */
 #endif
 
 
+/*
+ * Flags for FileBrowser call
+ */
+#define FB_READ		0x0001		/* Looking for a file to read. */
+#define FB_SAVE		0x0002		/* Looking for a file to save. */
+
+
 /*
  * number of keystrokes to delay removing an error message, or new mail
  * notification
