@@ -1,5 +1,5 @@
 #if !defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: context.c,v 4.35 1994/08/13 20:43:05 mikes Exp $";
+static char rcsid[] = "$Id: context.c,v 4.36 1994/10/11 16:37:08 mikes Exp $";
 #endif
 /*
  * Program:	Mailbox Context Management
@@ -657,8 +657,10 @@ long context_copy (context, stream, sequence, name)
       strcpy(&tmp[(s - context)+1], name);
   }
   else strcpy(tmp,name);		/* fully-qualified name */
-  if((*(s=tmp) == '{' || (*s == '*' && *(s+1) == '{')) && (s=strindex(s, '}')))
-    s++;				/* strip imap host name ? */
+  if((*(s=tmp) == '{' || (*s == '*' && *(s+1) == '{')) && (s=strindex(s,'}'))){
+      if(!*++s)				/* strip imap host name ? */
+	strcpy(s = tmp, "INBOX");	/* presume "inbox" ala c-client */
+  }
 
   return (s ? mail_copy(stream, sequence, s) : 0L);
 }
