@@ -1,5 +1,5 @@
 #if !defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: imap.c,v 4.48 1994/08/17 07:19:10 mikes Exp $";
+static char rcsid[] = "$Id: imap.c,v 4.49 1994/09/09 17:34:06 mikes Exp $";
 #endif
 /*----------------------------------------------------------------------
 
@@ -178,8 +178,12 @@ mm_log(string, errflg)
                                      errflg == PARSE ? "parse" : "babble",
                                      string));
 
-    if(ps_global->try_to_create		   /* hide if creating new folder */
-       || !strncmp(string, "[CLOSED]", 8)) /* *or* if error on dead stream */
+    /*
+     * don't display if creating new folder
+     * *or* if error on a stream known to be dead
+     */
+    if(ps_global->try_to_create
+       || (ps_global->noticed_dead_stream && !strncmp(string, "[CLOSED]", 8)))
       return;
 
     /*---- replace all "mailbox" with "folder" ------*/
