@@ -1,5 +1,5 @@
 #if !defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: mailindx.c,v 4.110 1994/08/18 02:14:17 mikes Exp $";
+static char rcsid[] = "$Id: mailindx.c,v 4.116 1994/10/11 01:17:12 mikes Exp $";
 #endif
 /*----------------------------------------------------------------------
 
@@ -151,16 +151,25 @@ static struct index_state {
  */
 #if	defined(DOS) && !defined(_WINDOWS)
 #define	SUB_CACHE_LEN	128
-static struct subcache {
-    long   size, cur;
-    char   *buf;
-    FILE  *ent;
-} *subcache = NULL;
+typedef struct {
+    short used;				/* whether entry's been used or not */
+    char  buf[SUB_CACHE_LEN];		/* actual subject string */
+} scache_ent;
+
+
+static struct scache {
+    short	last;			/* last one referenced */
+    long	size,			/* number of elements */
+		msgno[2];		/* which message in which entry */
+    scache_ent  ent[2];			/* subject and cache info for msg */
+    char       *cname;			/* file containing cache */
+    FILE       *cfile;
+} *scache = NULL;
 #else
-static struct subcache {
+static struct scache {
     long   size;
     char **ent;
-} *subcache = NULL;
+} *scache = NULL;
 #endif
 
 
@@ -184,7 +193,9 @@ int      i_cache_width();
 void	 msgno_flush_selected(MSGNO_S *, long);
 void	 second_subject_sort(long *, long, long);
 void	 init_subject_cache(long);
-char   **subject_cache_ent(long);
+int	 subject_cache_slot(long);
+char	*subject_cache_ent(long);
+char	*subject_cache_add(long, char *);
 void	 clear_subject_cache();
 #ifdef	DOS
 void     i_cache_hit(long);
@@ -203,8 +214,11 @@ int	 i_cache_width();
 void	 msgno_flush_selected();
 void	 second_subject_sort();
 void	 init_subject_cache();
-char   **subject_cache_ent();
+int	 subject_cache_slot();
+char	*subject_cache_ent();
+char	*subject_cache_add();
 void	 clear_subject_cache();
+
 #endif
 void	 sort_blip();
 
@@ -450,6 +464,10 @@ index_lister(state, cntxt, folder, stream, msgmap)
          * will have to change under UNIX if we want to do the same.
 	 */
 	while_waiting = build_header_cache;
+#ifdef	_WINDOWS
+	mswin_allowexit (MSWIN_EXIT_SENDCHAR, NULL,
+			 F_ON(F_USE_FK, ps_global) ? OPF3 : 'q');
+#endif
 #endif
 	ch = read_command();
 #ifdef	DOS
@@ -460,6 +478,9 @@ index_lister(state, cntxt, folder, stream, msgmap)
 	    clear_mfunc();
 	}
 #endif
+#ifdef	_WINDOWS
+	mswin_allowexit (MSWIN_EXIT_DENY, "Complete current operation", '\0');
+#endif
 #endif
 
         orig_ch = ch;
@@ -1151,10 +1172,10 @@ get_sub(mess)
      long mess;
 {
     ENVELOPE *e;
-    char *s, *s3, **ss;
+    char *s, *s3, *ss;
     int   l;
 
-    if(!*(ss = subject_cache_ent(mess))){
+    if(!(ss = subject_cache_ent(mess))){
 	e = mail_fetchstructure(ps_global->mail_stream, mess, NULL);
 	if(e && e->subject) {
 	    /* ---- Clean junk off the front of the subject ----*/
@@ -1175,18 +1196,18 @@ get_sub(mess)
 	    while(*s && (isspace(*s) || *s == '['))
 	      s++;
 
-	    if(*(*ss = cpystr(s))){
+	    if(*(ss = subject_cache_add(mess, s))){
 		/*----- Now, truncate junk off the back end of the subject---*/
-		for(s3 = NULL, s = *ss; *s; s++)	/* blast ws and ']' */
+		for(s3 = NULL, s = ss; *s; s++)	/* blast ws and ']' */
 		  s3 = (!isspace(*s) && *s != ']') ? NULL : (!s3) ? s : s3;
 
 		if(s3)
 		  *s3 = '\0';
 
-		if((l=(s3 ? s3 : s)-(*ss)) > 5 && !strucmp((*ss)+l-5,"(fwd)"))
-		  (*ss)[l-5] = '\0';
+		if((l=(s3 ? s3 : s)-ss) > 5 && !strucmp(ss+l-5,"(fwd)"))
+		  ss[l-5] = '\0';
 
-		for(s3 = NULL, s = *ss; *s; s++)	/* blast ws and ']' */
+		for(s3 = NULL, s = ss; *s; s++)	/* blast ws and ']' */
 		  s3 = (!isspace(*s) && *s != ']') ? NULL : (!s3) ? s : s3;
 
 		if(s3)
@@ -1194,12 +1215,12 @@ get_sub(mess)
 	    }
 	}
 	else
-	  *ss = cpystr("");
+	  ss = subject_cache_add(mess, "");
 
-	dprint(9, (debugfile, "SUB-GET-%s-GET-SUB\n", *ss))
+	dprint(9, (debugfile, "SUB-GET-%s-GET-SUB\n", ss))
     }
     else{
-	dprint(9, (debugfile, "SUB-HIT-%s-HIT-SUB\n", *ss))
+	dprint(9, (debugfile, "SUB-HIT-%s-HIT-SUB\n", ss))
     }
 
 #ifndef DOS
@@ -1208,7 +1229,7 @@ get_sub(mess)
         fetched_count++;
     }
 #endif
-    return(*ss);
+    return(ss);
 }
 
 
@@ -1488,6 +1509,7 @@ compare_arrival(a, b)
   ----*/
 #if	defined(DOS) && !defined(_WINDOWS)
 static FILE *second_sort_file;	/* Can't afford an array with DOS */
+static char *second_sort_file_name = NULL;
 #else
 static long *second_sort;	/* Array of message numbers */
 #endif
@@ -1560,7 +1582,8 @@ second_subject_sort(sort, max_msgno, total_max)
       return; 
 
 #if	defined(DOS) && !defined(_WINDOWS)
-    second_sort_file = tmpfile();
+    second_sort_file_name = temp_nam(NULL, "sf");
+    second_sort_file      = fopen(second_sort_file_name, "wb+");
 #else
     second_sort      = (long *) fs_get(sizeof(long) * (size_t)(total_max + 1));
 #endif
@@ -1619,6 +1642,8 @@ second_subject_sort(sort, max_msgno, total_max)
     /*---- Now clean up --------*/
 #if	defined(DOS) && !defined(_WINDOWS)
     fclose(second_sort_file);
+    unlink(second_sort_file_name);
+    fs_give((void **)&second_sort_file_name);
 #else    
     fs_give((void **)&second_sort);
 #endif
@@ -1665,10 +1690,9 @@ sort_current_folder()
         if(so == SortSubject || SortSubject2)		/* nmsgs cuz MN_EXLD */
 	  init_subject_cache(ps_global->mail_stream->nmsgs);
 
-#ifndef	DOS
-        if(mn_get_total(ps_global->msgmap) > 500)
-#endif
-            q_status_message1(0, 2, 4, "Sorting index by %s", sort_name(so));
+	if(mn_get_total(ps_global->msgmap) > 20)
+	  q_status_message2(0, 0, 0, "Sorting \"%s\" by %s",
+			  ps_global->cur_folder, sort_name(so));
 
         /*========= Sorting ================================================*/
 #ifndef DOS
@@ -1756,40 +1780,110 @@ void
 init_subject_cache(n)
     long n;
 {
-    subcache = (struct subcache *) fs_get(sizeof(struct subcache));
-    memset((void *)subcache, 0, sizeof(struct subcache));
-    subcache->size = n;
+    scache = (struct scache *) fs_get(sizeof(struct scache));
+    memset((void *)scache, 0, sizeof(struct scache));
+    scache->size = n;
 #if	defined(DOS) && !defined(_WINDOWS)
-    subcache->buf = fs_get(SUB_CACHE_LEN+1);
-    subcache->ent = tmpfile();
-    { long i;
-      for(i = 0L; i <= n; n++)
-	fwrite(subcache->buf, (size_t)SUB_CACHE_LEN, (size_t)1, subcache->ent);
+    scache->cname = temp_nam(NULL, "sc");
+    scache->cfile = fopen(scache->cname, "wb+");
+    /* BUG? go into non-caching mode if failure below? */
+    while(n-- >= 0L){
+	if(fwrite(&scache->ent[1], sizeof(scache_ent), (size_t)1,
+		  scache->cfile) != 1){
+	    sprintf(tmp_20k_buf, "subject cache: %s",
+		    error_description(errno));
+	    fatal(tmp_20k_buf);
+	}
     }
 #else
-    subcache->ent  = (char **)fs_get((size_t)(n+1L) * sizeof(char *));
-    memset((void *)subcache->ent, 0, (size_t)(n+1L) * sizeof(char *));
+    scache->ent  = (char **)fs_get((size_t)(n+1L) * sizeof(char *));
+    memset((void *)scache->ent, 0, (size_t)(n+1L) * sizeof(char *));
 #endif
 }
 
 
 /*----------------------------------------------------------------------
-    initialize the subject cache
+    return the subject of the given number if it's in the cache, otw NULL
   ----*/
-char **
+char *
 subject_cache_ent(n)
     long n;
 {
 #if	defined(DOS) && !defined(_WINDOWS)
-    fseek(subcache->ent, subcache->cur, 0);
-    fwrite((void *)subcache->buf, (size_t)SUB_CACHE_LEN, (size_t)1,
-	   subcache->ent);
-    subcache->cur = n;
-    fseek(subcache->ent, subcache->cur, 0);
-    fread((void *)subcache->buf,(size_t)SUB_CACHE_LEN,(size_t)1,subcache->ent);
-    return(&(subcache->buf));
+    char *buf;
+    int   i;
+
+    /* if the one we want's in core, return it */
+    buf = (scache->msgno[i=0] == n || scache->msgno[i=1] == n)
+	    ? scache->ent[i].buf : NULL;
+
+    /* else pick one of the slots, and fetch n's subject off disk */
+    if(!buf){
+	i = subject_cache_slot(n);
+	buf = scache->ent[i].used ? scache->ent[i].buf : NULL;
+    }
+
+    scache->last = i;
+    return(buf);
 #else
-    return(&subcache->ent[n]);
+    return(scache->ent[n]);
+#endif
+}
+
+
+/*----------------------------------------------------------------------
+    added the givin subject to the cache
+  ----*/
+char *
+subject_cache_add(n, s)
+    long n;
+    char *s;
+{
+#if	defined(DOS) && !defined(_WINDOWS)
+    int i;
+    
+    if(scache->msgno[i=0] != n && scache->msgno[i=1] != n)
+      i = subject_cache_slot(n);
+
+    scache->last = i;
+    scache->ent[i].used = 1;
+    scache->msgno[i] = n;
+    strncpy(scache->ent[i].buf, s, SUB_CACHE_LEN-1);
+    scache->ent[i].buf[SUB_CACHE_LEN-1] = '\0';
+    return(scache->ent[i].buf);
+#else
+    return(scache->ent[n] = cpystr(s));
+#endif
+}
+
+
+/*----------------------------------------------------------------------
+    pick the next slot, and fetch n's subject off disk
+  ----*/
+int
+subject_cache_slot(n)
+    long n;
+{
+#if	defined(DOS) && !defined(_WINDOWS)
+    int i;
+
+    i = scache->last ? 0 : 1;
+    if(scache->msgno[i]){
+	fseek(scache->cfile, scache->msgno[i] * sizeof(scache_ent), 0);
+	fwrite(&scache->ent[i], sizeof(scache_ent), (size_t)1,
+	       scache->cfile);
+    }
+
+    scache->msgno[i] = n;
+    if(fseek(scache->cfile, scache->msgno[i] * sizeof(scache_ent), 0)
+       || fread(&scache->ent[i], sizeof(scache_ent), (size_t)1,
+		scache->cfile) != 1){
+	sprintf(tmp_20k_buf, "Can't access subject cache: ",
+		error_description(errno));
+	fatal(tmp_20k_buf);
+    }
+
+    return(i);
 #endif
 }
 
@@ -1800,17 +1894,18 @@ subject_cache_ent(n)
 void
 clear_subject_cache()
 {
-    if(subcache){
+    if(scache){
 #if	defined(DOS) && !defined(_WINDOWS)
-	fs_give((void**)&subcache->buf);
-	fclose(subcache->ent);
+	fclose(scache->cfile);
+	unlink(scache->cname);
+	fs_give((void **)&scache->cname);
 #else
-	for(; subcache->size; subcache->size--)
-	  if(subcache->ent[subcache->size])
-	    fs_give((void **)&subcache->ent[subcache->size]);
+	for(; scache->size; scache->size--)
+	  if(scache->ent[scache->size])
+	    fs_give((void **)&scache->ent[scache->size]);
 
 #endif
-	fs_give((void **)&subcache);
+	fs_give((void **)&scache);
     }
 }
 
@@ -1941,16 +2036,21 @@ msgno_exclude(stream, msgs)
 {
     long	  i, j, span;
     int		  shift;
-    MESSAGECACHE *mc;
 
     if(!msgs || msgs->max_msgno < 1L)
       return;
 
-    FETCH_ALL_FLAGS(stream);		/* make sure all flags up to date */
+    /*
+     * With 3.91 we're using a new strategy for finding and operating
+     * on all the messages with deleted status.  The idea is to do a
+     * mail_search for deleted messages so the elt's "searched" bit gets
+     * set, and then to scan the elt's for them and set our local bit
+     * to indicate they're excluded...
+     */
+    (void)count_flagged(stream, "DELETED");
 
-    i = 1L; 
-    while(i <= msgs->max_msgno){
-	if((mc = mail_elt(stream, mn_m2raw(msgs, i))) && mc->deleted){
+    for(i = 1L; i <= msgs->max_msgno; ){
+	if(mail_elt(stream, mn_m2raw(msgs, i))->searched){
 	    /*--- clear all flags to keep our counts consistent  ---*/
 	    set_lflag(stream, msgs, i, (MN_HIDE|MN_SLCT), 0);
 	    set_lflag(stream, msgs, i, MN_EXLD, 1); /* mark excluded */
@@ -2229,10 +2329,10 @@ msgno_in_sort(msgs, n)
  * the only changes here would be to pass the ps pointer around
  */
 static struct index_cache {
-   void	    *cache;			/* pointer to cache */
-   char	    *name;			/* pointer to cache name */
-   long      num;			/* number of last index in cache */
-   unsigned  size;			/* size of each index line       */
+   void	  *cache;				/* pointer to cache */
+   char	  *name;				/* pointer to cache name */
+   long    num;					/* # of last index in cache */
+   size_t  size;				/* size of each index line  */
 } icache = { NULL, 0L, 0 };
   
 /*
@@ -2286,27 +2386,27 @@ i_cache_size(indx)
     long         indx;
 {
     long j;
-    int  newsize = sizeof(HLINE_S)
+    size_t  newsize = sizeof(HLINE_S)
 		     + (max(ps_global->ttyo->screen_cols, 80) * sizeof(char));
 
-    if(j = (newsize % sizeof(int))) 		/* alignment hack */
-      newsize += (sizeof(int) - j);
+    if(j = (newsize % sizeof(long)))		/* alignment hack */
+      newsize += (sizeof(long) - (size_t)j);
 
     if(icache.size != newsize){
 	clear_index_cache();			/* clear cache, start over! */
-	icache.size = (unsigned) newsize;
+	icache.size = newsize;
     }
 
     if(indx > (j = icache.num - 1L)){		/* make room for entry! */
-	int     tmplen = (int) icache.size;
+	size_t  tmplen = icache.size;
 	char   *tmpline;
 
 	while(indx >= icache.num)
 	  icache.num += IC_SIZE;
 
 #ifdef	DOS
-	tmpline = fs_get((size_t)tmplen);
-	memset(tmpline, 0, (size_t)tmplen);
+	tmpline = fs_get(tmplen);
+	memset(tmpline, 0, tmplen);
 	if(icache.cache == NULL){
 	    if(!icache.name)
 	      icache.name = temp_nam(NULL, "pi");
@@ -2317,8 +2417,7 @@ i_cache_size(indx)
 	    }
 
 	    for(j = 0; j < icache.num; j++){
-	        if(fwrite(tmpline,(size_t)tmplen,
-			  (size_t)1,(FILE *)icache.cache) != 1)
+	        if(fwrite(tmpline,tmplen,(size_t)1,(FILE *)icache.cache) != 1)
 		  fatal("Can't write index cache in resize");
 
 		if(j%ICC_SIZE == 0){
@@ -2333,8 +2432,7 @@ i_cache_size(indx)
 	    fseek((FILE *)icache.cache, 0L, 2);		/* seek to end */
 
 	    for(;j < icache.num; j++){
-	        if(fwrite(tmpline,(size_t)tmplen,
-			(size_t)1,(FILE *)icache.cache) != 1)
+	        if(fwrite(tmpline,tmplen,(size_t)1,(FILE *)icache.cache) != 1)
 		  fatal("Can't write index cache in resize");
 
 		if(j%ICC_SIZE == 0){
