@@ -1,5 +1,5 @@
 #if !defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: mailpart.c,v 4.7 1994/08/11 20:16:04 mikes Exp $";
+static char rcsid[] = "$Id: mailpart.c,v 4.15 1994/10/05 22:13:25 hubert Exp $";
 #endif
 /*----------------------------------------------------------------------
 
@@ -337,6 +337,7 @@ attachment_screen(ps, msgmap)
 	    {int   rc, found = 0;
 	     char *result = NULL, buf[64];
 	     static char last[64], tmp[64];
+	     HelpType help;
 
 	     ps->mangled_footer = 1;
 	     buf[0] = '\0';
@@ -344,9 +345,12 @@ attachment_screen(ps, msgmap)
 		     (last[0]) ? "[" : "",
 		     (last[0]) ? last : "",
 		     (last[0]) ? "]" : "");
-	     while(1) {
-		 rc = optionally_enter(buf,-3,0,63,1,0,tmp,NULL,NO_HELP,0);
-		 if(rc == 0 || rc == 1 || !buf[0]){
+	     help = NO_HELP;
+	     while(1){
+		 rc = optionally_enter(buf,-3,0,63,1,0,tmp,NULL,help,0);
+		 if(rc == 3)
+		   help = help == NO_HELP ? h_attach_index_whereis : NO_HELP;
+		 else if(rc == 0 || rc == 1 || !buf[0]){
 		     if(rc == 0 && !buf[0] && last[0])
 		       strcpy(buf, last);
 
@@ -761,9 +765,14 @@ save_attachment(qline, msgno, a)
                 full_filename + ((r > 20) ? r - 20 : 0));
 
 	switch(radio_buttons(prompt_buf, -3, 0, access_opts, 'a',
-			    'x', 0, NO_HELP)){
+			    'x', 0, NO_HELP, 0)){
 	  case 'o' :
-	    unlink(full_filename);		/* BUG: breaks links */
+	    if(unlink(full_filename) < 0){	/* BUG: breaks links */
+		q_status_message2(1, 3, 5, "\007Error deleting old %s: %s",
+				  full_filename, error_description(errno));
+		return;
+	    }
+
 	    break;
 
 	  case 'a' :
@@ -836,7 +845,7 @@ display_attachment(msgno, a)
     if(!a->can_display) {
         /*----- Can't display this type ------*/
 	if(!mime_can_display(a->body->type, a->body->subtype,
-			     a->body->parameter))
+			     a->body->parameter, (int *)NULL))
 	  q_status_message3(1, 3, 5,
 		      "\007Don't know how to display attachment format %s%s%s",
                       body_type_names(a->body->type),
@@ -850,13 +859,12 @@ display_attachment(msgno, a)
 
         return(1);
     }
-    else if(a->body->type == TYPETEXT) {
-	display_text_att(msgno, a);
-	ps_global->mangled_screen = 1;
-        return(0);
-    }
-    else if(a->body->type == TYPEMESSAGE){
-	display_msg_att(msgno, a);
+    else if(!a->use_external_viewer){
+	if(a->body->type == TYPETEXT)
+	  display_text_att(msgno, a);
+	else if(a->body->type == TYPEMESSAGE)
+	  display_msg_att(msgno, a);
+
 	ps_global->mangled_screen = 1;
 	return(0);
     }
@@ -907,13 +915,14 @@ run_viewer(image_file, body)
      char *image_file;
      BODY *body;
 {
-    char *cmd = NULL;
+    char *cmd            = NULL;
+    int   needs_terminal = 0;
 
     q_status_message(0,0,3,"Displaying attachment...");
     display_message('x');
 
-    if(cmd = mailcap_build_command(body, image_file)){
-	exec_mailcap_cmd(cmd, image_file);
+    if(cmd = mailcap_build_command(body, image_file, &needs_terminal)){
+	exec_mailcap_cmd(cmd, image_file, needs_terminal);
 	fs_give((void **)&cmd);
     }
     else
@@ -1006,12 +1015,23 @@ display_msg_att(msgno, a)
 
     gf_set_so_writec(&pc, store);
 
-    format_envelope(a->body->contents.msg.env, pc);
-    gf_puts(NEWLINE, pc);
-    if((a+1)->body->type == TYPETEXT)
-      (void) decode_text(a+1, msgno, pc, QStatus, 1);
+    if(a->body->subtype && strucmp(a->body->subtype, "rfc822") == 0){
+	format_envelope(a->body->contents.msg.env, pc);
+	gf_puts(NEWLINE, pc);
+	if((a+1)->body && (a+1)->body->type == TYPETEXT)
+	  (void) decode_text(a+1, msgno, pc, QStatus, 1);
+	else
+	  gf_puts("[Can't display first non-text segment]", pc);
+    }
+    else if(a->body->subtype 
+	    && strucmp(a->body->subtype, "external-body") == 0) {
+	gf_puts("This part is not included and can be fetched as follows:",pc);
+	gf_puts(NEWLINE, pc);
+	gf_puts(NEWLINE, pc);
+	gf_puts(display_parameters(a->body->parameter), pc);
+    }
     else
-      gf_puts("[Can't display first non-text segment]", pc);
+      (void) decode_text(a, msgno, pc, QStatus, 1);
 
     scrolltool(so_text(store), "ATTACHED MESSAGE", (int *)NULL,
 	       AttachText, src, a);
@@ -1034,6 +1054,7 @@ pipe_attachment(msgno, a)
   char    prompt[80] ;
   int     done = 0 ;
   PIPE_S *syspipe;
+  HelpType help;
 
   if(ps_global->restricted){
       q_status_message(0,2,2 ,"\007Pine demo can't pipe attachments");
@@ -1041,17 +1062,20 @@ pipe_attachment(msgno, a)
   }
 
   sprintf(prompt, "Pipe attachment %s to : ", a->number) ;
+  help = NO_HELP;
   while(!done) {
     flush_status_messages() ;
     switch(optionally_enter(ps_global->pipe_command, -3, 0, MAXPATH, 1, 0, 
-			    prompt, NULL, NO_HELP, 0)){
+			    prompt, NULL, help, 0)){
     case -1 :
       q_status_message(0,2,2 ,"\007Internal problem encountered");
       done++;
       break;
       
     case 0 :
-      if(syspipe = open_system_pipe(ps_global->pipe_command,
+      if(ps_global->pipe_command[0] == '\0'){
+	  q_status_message(0, 1, 2, "Pipe command cancelled");
+      }else if(syspipe = open_system_pipe(ps_global->pipe_command,
 				    &resultfilename, PIPE_USER)){
 	  gf_io_t pc;		/* wire up a generic putchar */
 	  gf_set_writec(&pc, syspipe->ifile, 0L, FileStar);
@@ -1062,7 +1086,7 @@ pipe_attachment(msgno, a)
 	  if(err)
 	    q_status_message1(1,3,5,"\007Error detaching for pipe: %s",err);
 
-	  close_system_pipe(&syspipe);
+	  close_system_pipe(&syspipe, PIPE_USER);
 	  if(err)
 	    unlink(resultfilename);
 	  else
@@ -1079,9 +1103,12 @@ pipe_attachment(msgno, a)
       q_status_message(0,2,2 ,"Pipe cancelled");
       done++;
       break;
+
+    case 3 :
+      help = (help == NO_HELP) ? h_pipe_attach : NO_HELP;
+      break;
       
     case 2 :                              /* no place to escape to */
-    case 3 :                              /* no help yet   */
     case 4 :                              /* can't suspend */
       default :
 	break;   
@@ -1114,6 +1141,7 @@ detach(stream, msg_no, body, part_no, len, pc, aux_filters)
     SourceType     src = CharStar;
     static char    err_string[100];
 #ifdef	DOS
+    char	  *tmpfile_name = NULL;		/* name our own tmp file */
     extern unsigned char  *xlate_to_codepage;
 #endif
 
@@ -1121,11 +1149,21 @@ detach(stream, msg_no, body, part_no, len, pc, aux_filters)
 
 #ifdef	DOS
     if(body->size.bytes > MAX_MSG_INCORE || !strcmp(stream->dtb->name,"nntp")){
-	src      = FileStar;
-	mailgets = dos_gets;
+	src = FileStar;
+	if(!(tmpfile_name = temp_nam(NULL, "dt"))
+	   || !(append_file = fopen(tmpfile_name, "w+b"))){
+	    if(tmpfile_name)
+	      fs_give((void **)&tmpfile_name);
+
+	    sprintf(err_string, "Can't detach!  Temp file %s",
+		    error_description(errno));
+	    return(err_string);
+	}
+
+	mail_parameters(stream, SET_GETS, (void *)dos_gets);
     }
     else
-      mailgets = NULL;
+      mail_parameters(stream, SET_GETS, (void *)NULL);
 #endif	/* DOS */
 
     /*
@@ -1133,16 +1171,12 @@ detach(stream, msg_no, body, part_no, len, pc, aux_filters)
      */
     contents = mail_fetchbody(stream, msg_no, part_no, &length);
     if(contents == NULL) {
-#ifdef	DOS
-        mailgets = NULL;
-#endif
 	sprintf(err_string, "Unable to access body part %s", part_no);
-	if (len != (long *)NULL)
-	    *len = 0;
-	return(err_string);
+	rv = 0L;
+	goto fini;
     }
 
-    rv = (length) ? length : 1;
+    rv = (length) ? length : 1L;
 
     gf_filter_init();			/* prepare to filter it! */
 
@@ -1185,7 +1219,8 @@ detach(stream, msg_no, body, part_no, len, pc, aux_filters)
 	 * matches what the user's defined as our charset,
 	 * translate it...
 	 */
-	if(mime_can_display(body->type, body->subtype, body->parameter)
+	if(mime_can_display(body->type, body->subtype, body->parameter,
+								(int *)NULL)
 	   && xlate_to_codepage){
 	    gf_translate_opt(xlate_to_codepage, 256);
 	    gf_link_filter(gf_translate);
@@ -1201,36 +1236,37 @@ detach(stream, msg_no, body, part_no, len, pc, aux_filters)
     gf_set_readc(&gc,
 #ifdef	DOS
 		 /*
-		  * If we fetched this to a file, 
-		  * make sure we set up the storage
-		  * object's getchar function accordingly...
+		  * If we fetched this to a file, make sure we set
+		  * up the storage object's getchar function accordingly...
 		  */
-		 (src == FileStar) ? (char *)append_file :
+		 (src == FileStar) ? (void *)append_file :
 #endif
-		 contents,
-		 length, src);
+		 contents, length, src);
 
     if(status = gf_pipe(gc, pc)) {
 	sprintf(err_string, "Formatting error: %s", status);
         rv = 0L;
     }
 
+  fini :
+
 #ifdef	DOS
     /*
-     * free up file descriptor, and return tmpfile opened by
-     * dos_gets and returned via the global "append_file"...
+     * free up file pointer, and delete tmpfile opened for
+     * dos_gets.  Again, we can't use DOS' tmpfile() as it writes root
+     * sheesh.
      */
     if(src == FileStar){
-	if(append_file)
-	  fclose(append_file);
-
+	fclose(append_file);
 	append_file = NULL;
-	mailgets    = NULL;
+	unlink(tmpfile_name);
+	fs_give((void **)&tmpfile_name);
+	mail_parameters(stream, SET_GETS, (void *)NULL);
 	mail_gc(stream, GC_TEXTS);
     }
 #endif
 
-    if (len != (long *)NULL)
+    if (len)
       *len = rv;
 
     return((err_string[0] == '\0') ? NULL : err_string);
