@@ -1,5 +1,5 @@
 #if !defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: mailview.c,v 4.124 1994/08/22 22:57:23 mikes Exp $";
+static char rcsid[] = "$Id: mailview.c,v 4.132 1994/10/10 20:00:43 mikes Exp $";
 #endif
 /*----------------------------------------------------------------------
 
@@ -99,7 +99,6 @@ int	   make_file_index();
 char      *show_multipart(MESSAGECACHE *, int);
 MimeShow   mime_show(BODY *);
 char      *part_desc(char *,BODY *, int);
-char      *display_parameters(PARAMETER *);
 SCROLL_S  *new_scroll_state();
 int	   search_text(int, int, char *);
 void	   format_scroll_text(void);
@@ -124,7 +123,6 @@ int	   make_file_index();
 char      *show_multipart();
 MimeShow   mime_show();
 char      *part_desc();
-char      *display_parameters();
 SCROLL_S  *new_scroll_state();
 int	   search_text();
 void	   format_scroll_text();
@@ -166,8 +164,12 @@ mail_view_screen(ps)
     dprint(1, (debugfile, "\n\n  -----  MAIL VIEW  -----\n"));
 
     if(mn_get_total(ps->msgmap) <= 0 || ps->mail_stream == NULL) {
-        q_status_message(0, 1, 3, "No messages to read!");
-        ps->next_screen = ps->prev_screen;
+        q_status_message(0, 0, 3, "No messages to read!");
+	/*
+	 * Don't just come back here if there aren't any messages!
+	 */
+	ps->next_screen = (ps->prev_screen != mail_view_screen)
+			    ? ps->prev_screen : mail_index_screen;
 	return;
     }
 
@@ -278,7 +280,7 @@ describe_mime(body, prefix, num, should_show)
 {
     PART      *part;
     char       numx[50], string[200], *description;
-    int        n, alt_to_show, has_name;
+    int        n, alt_to_show, has_name, use_viewer;
     ATTACH_S  *a;
     MimeShow   alts[50]; /* BUG -- no bounds checking on this */
     PARAMETER *param;
@@ -340,8 +342,9 @@ describe_mime(body, prefix, num, should_show)
 	 * type/subtype, *AND* that we can decode it if needed...
 	 */
         a->can_display = (mime_can_display(body->type, body->subtype,
-                                          body->parameter)
+                                          body->parameter, &use_viewer)
 			  && body->encoding < ENCOTHER);
+	a->use_external_viewer = use_viewer;
 	for(param = a->body->parameter; 
 	    param != NULL && strucmp(param->attribute,"name") != 0;
 	    param = param->next)
@@ -544,7 +547,8 @@ mime_show(body)
 
     switch(body->type) {
       default:
-        return(mime_can_display(body->type,body->subtype,body->parameter)== 1 ?
+        return(mime_can_display(body->type,
+		    body->subtype,body->parameter,(int *)NULL)== 1 ?
                ShowAll:
                ShowNone);
 
@@ -636,10 +640,9 @@ format_message(msgno, env, body, flgs, pc)
 
 	while(*h){
 	    /*
-	     * If we're doing this for a printout, we want to change
-	     * the network newlines into the local format...
+	     * convert network newlines to local convention...
 	     */
-	    if((flgs&FM_DO_PRINT) && *h == '\015' && *(h+1) == '\012'){
+	    if(*h == '\015' && *(h+1) == '\012'){
 		if(gf_puts(NEWLINE, pc))
 		  h += 2;
 		else
@@ -665,11 +668,7 @@ format_message(msgno, env, body, flgs, pc)
 	    char *p = h;
 
 	    while(*p){
-		/*
-		 * If we're doing this for a printout, we want to change
-		 * the network newlines into the local format...
-		 */
-		if((flgs&FM_DO_PRINT) && *p == '\015' && *(p+1) == '\012'){
+		if(*p == '\015' && *(p+1) == '\012'){
 		    if(gf_puts(NEWLINE, pc))
 		      p += 2;
 		    else
@@ -688,13 +687,23 @@ format_message(msgno, env, body, flgs, pc)
               so we just show the text here. Hopefully the 
               message isn't a MIME message 
           ---*/
-	void            *text2;
+	void *text2;
 #ifdef	DOS
+	char *append_file_name;
+
 	/* for now, always fetch to disk.  This could be tuned to
 	 * check for message size, then decide to deal with it on disk...
 	 */
-	mailgets    = dos_gets;
-	append_file = NULL;		/* let dos_gets create tmp file */
+	mail_parameters(ps_global->mail_stream, SET_GETS, (void *)dos_gets);
+	if(!(append_file_name = temp_nam(NULL, "pv"))
+	   || !(append_file = fopen(append_file_name, "w+b"))){
+	    if(append_file_name)
+	      fs_give((void **)&append_file_name);
+
+	    q_status_message1(0,2,2,"Can't make temp file: %s",
+			      error_description(errno));
+	    return(0);
+	}
 #endif
 
         if(text2 = (void *)mail_fetchtext(ps_global->mail_stream, msgno)){
@@ -713,15 +722,18 @@ format_message(msgno, env, body, flgs, pc)
 		if(!gf_puts(tmp_20k_buf, pc))
 		  goto write_error;
 	    }
-#ifdef	DOS
-	    if(append_file){
-		fclose(append_file);
-		append_file = NULL;
-		mail_gc(ps_global->mail_stream, GC_TEXTS);
-	    }
-#endif
 	}
-	else{
+
+#ifdef	DOS
+	fclose(append_file);			/* clean up tmp file */
+	append_file = NULL;
+	unlink(append_file_name);
+	fs_give((void **)&append_file_name);
+	mail_gc(ps_global->mail_stream, GC_TEXTS);
+	mail_parameters(ps_global->mail_stream, SET_GETS, (void *)NULL);
+#endif
+
+	if(!text2){
 	    if(!gf_puts(NEWLINE, pc)
 	       || !gf_puts("    [ERROR fetching text of message]", pc)
 	       || !gf_puts(NEWLINE, pc)
@@ -729,12 +741,9 @@ format_message(msgno, env, body, flgs, pc)
 	      goto write_error;
 	}
 
-#ifdef	DOS
-	mailgets = NULL;		/* unset special switch */
-#endif
 	return(1);
     }
-        
+
     if(flgs&FM_NEW_MESS) {
         zero_atmts(ps_global->atmts);
         describe_mime(body, "", 1, 1);
@@ -1469,11 +1478,14 @@ scrolltool(text, title, pages, style, source, att)
 	/*================ Get command and validate =====================*/
 #ifdef	_WINDOWS
 	mswin_allowcopy(mswin_readscrollbuf);
+	mswin_allowexit (MSWIN_EXIT_SENDCHAR, NULL,
+			 F_ON(F_USE_FK, ps_global) ? OPF3 : 'q');
 #endif
         ch = read_command();
         orig_ch = ch;
 #ifdef	_WINDOWS
 	mswin_allowcopy(NULL);
+	mswin_allowexit (MSWIN_EXIT_DENY, "Complete current operation", '\0');
 #endif
 
         if(ch < 0x0100 && isupper(ch))
@@ -1878,7 +1890,9 @@ whereis:
 		else {		/*----------- Unknown command -------*/
 unknown:
 		    bogus_command(orig_ch,
-				  F_ON(F_USE_FK,ps_global) ? "F1" : "?");
+				  (style == MainHelpText || style == HelpText)
+				    ? NULL
+				    : F_ON(F_USE_FK,ps_global) ? "F1" : "?");
     	        }
             }
             break;
@@ -2332,8 +2346,8 @@ int line;
 	 * will have limits, so maybe a disk file that is an array
 	 * of indexes is the answer.
 	 */
-	fseek((FILE *) st->findex, (size_t)(line) * sizeof(long), 0);
-	if(fread(&x, sizeof(long), (size_t)1, (FILE *)st->findex) != 1){
+	fseek(st->findex, (size_t)(line) * sizeof(long), 0);
+	if(fread(&x, sizeof(long), (size_t)1, st->findex) != 1){
 	    return;
 	}
 
@@ -2372,7 +2386,7 @@ make_file_index()
 	if(!st->fname)
 	  st->fname = temp_nam(NULL, "pi");
 
-	if((st->findex = (void *)fopen(st->fname,"w+b")) == NULL)
+	if((st->findex = fopen(st->fname,"w+b")) == NULL)
 	  return(0);
     }
     else
@@ -2620,6 +2634,62 @@ xmail_fetchheader_lines(stream, msgno, fields)
 }
 
 
+/*----------------------------------------------------------------------
+   Fetch everything except the requested header fields from given msgno
+
+   Args: stream -- mail stream of open folder
+         msgno -- number of message to get header lines from
+         fields -- array of pointers to UNdesired fields
+
+   Returns: allocated string containing NON-matching header lines,
+	    NULL on error.
+ ----*/
+char *
+xmail_fetchheader_lines_not(stream, msgno, fields)
+     MAILSTREAM  *stream;
+     long         msgno;
+     char       **fields;
+{
+    int   i, old_prefetch;
+    char *p, *m, *h = NULL, *match = NULL, tmp[MAILTMPLEN];
+    extern int find_field();
+
+    old_prefetch = (int) mail_parameters(stream, SET_PREFETCH, NULL);
+    h		 = mail_fetchheader(stream, msgno);
+    mail_parameters(stream, SET_PREFETCH, (void *) old_prefetch);
+
+    if(!h)
+      return(NULL);				/* fetchheader return error? */
+
+    while(find_field(&h, tmp)){
+	for(i = 0; fields[i] && strucmp(tmp, fields[i]); i++)
+	  ;
+
+	if(!fields[i]){				/* interesting field */
+	    /*
+	     * Hold off allocating space for matching fields until
+	     * we at least find one to copy...
+	     */
+	    if(!match)
+	      match = m = fs_get(strlen(h) + strlen(tmp) + 1);
+
+	    sstrcpy(&m, tmp);			/* copy field name */
+	    while(*h && (*m++ = *h++))		/* header includes colon */
+	      if(*(m-1) == '\n' && (*h == '\r' || !isspace(*h)))
+		break;
+
+	    *m = '\0';				/* tie off match string */
+	}
+	else{					/* no match, pass this field */
+	    while(*h && !(*h++ == '\n' && (*h == '\r' || !isspace(*h))))
+	      ;
+	}
+    }
+
+    return(match ? match : cpystr(""));
+}
+
+
 int
 find_field(h, tmp)
      char **h;
