@@ -1,4 +1,4 @@
-# $Id: makefile.hpp,v 4.17 1994/07/25 22:21:20 mikes Exp $
+# $Id: makefile.hpp,v 4.19 1994/10/05 06:14:29 hubert Exp $
 #
 #            T H E    P I N E    M A I L   S Y S T E M
 #
@@ -148,7 +148,7 @@ osdep/os-hpp.c:	osdep/bld_path osdep/canacces osdep/canonicl \
 		osdep/expnfldr osdep/fgetpos osdep/filesize osdep/fltrname \
 		osdep/fnexpand osdep/header osdep/hostname.una \
 		osdep/jobcntrl osdep/lstcmpnt osdep/mimedisp osdep/pipe \
-		osdep/play_snd.non osdep/pw_stuff osdep/readfile \
+		osdep/pw_stuff osdep/readfile osdep/debuging \
 		osdep/rename osdep/srandom.dum osdep/tempfile osdep/tempnam \
 		osdep/sendmail osdep/execview osdep/writ_dir osdep/sunquota \
 		osdep/os-hpp.ic
