@@ -1,5 +1,5 @@
 /*----------------------------------------------------------------------
-  $Id: os-a32.h,v 5.18 1994/08/23 02:47:02 hubert Exp $
+  $Id: os-a32.h,v 5.20 1994/10/12 01:47:49 mikes Exp $
 
             T H E    P I N E    M A I L   S Y S T E M
 
@@ -206,7 +206,6 @@
 /* char *strcpy(), *strncpy();  /* usually in string.h */
 
 
-/*------- Include types.h if you have it -------------------------------*/
 #include <sys/types.h>
 #endif /*****************************************************************/
 
@@ -264,6 +263,11 @@ extern void (*sigset())();       /* declare sigset if not declared already */
 /* #include <sys/ptem.h>   /* for struct winsize */
 
 
+/*------------ AIX 3.2 needs this for types passed select() ------------*/
+#include <sys/select.h>
+
+
+/*------- Include types.h if you have it -------------------------------*/
 
 /*-- What argument does wait(2) take? Define this if it is a union -----*/
 /* #define HAVE_WAIT_UNION  /* the arg to wait is a union wait * */
@@ -287,6 +291,10 @@ extern void (*sigset())();       /* declare sigset if not declared already */
 					   and bcc header lines.      */
 
 
+/*--------- Program employed by users to change their password ---------*/
+#define	PASSWD_PROG	"/bin/passwd"
+
+
 /*-------------- A couple constants used to size arrays ----------------*/
 #include <sys/param.h>          /* Get it from param.h if available */
 #undef MAXPATH                  /* Sometimes defined in param.h differently */
