@@ -1,5 +1,5 @@
 /*----------------------------------------------------------------------
-  $Id: os-lnx.h,v 1.17 1994/08/23 02:47:02 hubert Exp $
+  $Id: os-lnx.h,v 1.20 1994/10/11 22:51:59 mikes Exp $
 
             T H E    P I N E    M A I L   S Y S T E M
 
@@ -223,7 +223,7 @@
 /*----------------- time.h ---------------------------------------------*/
 #include <time.h> /* */
 /* plain time.h isn't enough on some systems */
-/* #include <sys/time.h>  /* For struct timeval usually in time.h */ 
+#include <sys/time.h>  /* For struct timeval usually in time.h */ 
 
 
 
@@ -232,7 +232,6 @@
 /* #include <sys/signal.h>  /* only one or the other */
 
 #define SIGSYS SIGUNUSED
-#define SIGBUS SIGUNUSED
 #define SIGEMT SIGUNUSED
 
 #define SigType void     /* value returned by signal() is void * */
@@ -289,6 +288,10 @@
 					   and bcc header lines.      */
 
 
+/*--------- Program employed by users to change their password ---------*/
+#define	PASSWD_PROG	"/bin/passwd"
+
+
 /*-------------- A couple constants used to size arrays ----------------*/
 #include <sys/param.h>          /* Get it from param.h if available */
 #undef MAXPATH                  /* Sometimes defined in param.h differently */
