@@ -1,5 +1,5 @@
 /*----------------------------------------------------------------------
-  $Id: os-osf.h,v 1.16 1994/08/23 02:47:02 hubert Exp $
+  $Id: os-osf.h,v 1.17 1994/10/10 22:29:35 mikes Exp $
 
             T H E    P I N E    M A I L   S Y S T E M
 
@@ -289,6 +289,10 @@ int fgetpos(); int fsetpos();
 					   and bcc header lines.      */
 
 
+/*--------- Program employed by users to change their password ---------*/
+#define	PASSWD_PROG	"/bin/passwd"
+
+
 /*-------------- A couple constants used to size arrays ----------------*/
 #include <sys/param.h>          /* Get it from param.h if available */
 #undef MAXPATH                  /* Sometimes defined in param.h differently */
