@@ -1,5 +1,5 @@
 /*----------------------------------------------------------------------
-  $Id: os-sv4.h,v 5.20 1994/08/23 02:47:02 hubert Exp $
+  $Id: os-sv4.h,v 5.21 1994/10/10 22:29:35 mikes Exp $
 
             T H E    P I N E    M A I L   S Y S T E M
 
@@ -288,6 +288,10 @@ extern void (*sigset())();       /* declare sigset if not declared already */
 					   and bcc header lines.      */
 
 
+/*--------- Program employed by users to change their password ---------*/
+#define	PASSWD_PROG	"/bin/passwd"
+
+
 /*-------------- A couple constants used to size arrays ----------------*/
 #include <sys/param.h>          /* Get it from param.h if available */
 #undef MAXPATH                  /* Sometimes defined in param.h differently */
