@@ -1,5 +1,5 @@
 #if !defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: other.c,v 4.71 1994/08/23 23:39:42 mikes Exp $";
+static char rcsid[] = "$Id: other.c,v 4.85 1994/10/08 00:28:43 hubert Exp $";
 #endif
 /*----------------------------------------------------------------------
 
@@ -121,6 +121,7 @@ void	 free_confline(CONF_S **);
 CONF_S	*first_confline(CONF_S *);
 void	 config_add_list(struct pine *, CONF_S **, char **, char ***);
 void	 config_del_list_item(CONF_S **, char ***);
+int      offer_to_fix_pinerc(struct pine *);
 
 #else
 void	 draw_klocked_body();
@@ -138,6 +139,7 @@ void	 free_confline();
 CONF_S	*first_confline();
 void	 config_add_list();
 void	 config_del_list_item();
+int      offer_to_fix_pinerc();
 #endif
 
 static char *klockin, *klockame;
@@ -207,10 +209,8 @@ lock_keyboard()
 			       "Enter password to LOCK keyboard : ", NULL,
                                help, 0);
 
-	if(rc == 3) {
-	    help = NO_HELP;		/* BUG: more help than from kl_body? */
-	    continue;
-        }
+	if(rc == 3)
+	  help = help == NO_HELP ? h_kb_lock : NO_HELP;
 	else if(rc == 1 || inpasswd[0] == '\0'){
 	    q_status_message(0, 0, 2, "Lock keyboard cancelled");
 	    return(-1);
@@ -312,6 +312,7 @@ static struct key_menu config_yesno_keymenu = {sizeof(config_yesno_keys)/(sizeof
 			  && (V) != &(P)->vars[V_LAST_TIME_PRUNE_QUESTION]  \
 			  && (V) != &(P)->vars[V_FONT_NAME]		    \
 			  && (V) != &(P)->vars[V_FONT_SIZE]		    \
+			  && (V) != &(P)->vars[V_FONT_STYLE]		    \
 			  && (V) != &(P)->vars[V_WINDOW_POSITION]	    \
 			  && (V) != &(P)->vars[V_LAST_VERS_USED])
 #else
@@ -375,6 +376,16 @@ option_screen(ps)
     OPT_SCREEN_S  screen;
     NAMEVAL_S	 *f;
 
+    mailcap_free(); /* free resources we won't be using for a while */
+
+    if(ps->fix_fixed_warning){
+	set_titlebar(CONFIG_SCREEN_TITLE, ps->mail_stream,
+		     ps->context_current,
+		     ps->cur_folder, ps->msgmap, 1, FolderName, 0, 0);
+        if(offer_to_fix_pinerc(ps))
+          write_pinerc(ps);
+    }
+
     screen.current = screen.top_line = NULL;
 
     /*
@@ -585,7 +596,8 @@ option_screen(ps)
 			 */
 			current->varmem = i + (j * EndofList);
 			sprintf(tmp, "(%c)  %s%-*s%*s",
-				(ps->def_sort == i && ps->def_sort_rev == j)
+				(ps->def_sort == (SortOrder) i
+						      && ps->def_sort_rev == j)
 				  ? R_SELD : ' ',
 				(j) ? "Reverse " : "",
 				lv, sort_name(i),
@@ -693,7 +705,7 @@ option_screen(ps)
 	    }
 	}
 #endif
-	else if(vtmp == &ps->vars[V_USE_ONLY_DOMAIN_NAME]){ /* radio case */
+	else if(vtmp == &ps->vars[V_USE_ONLY_DOMAIN_NAME]){ /* yesno case */
 	    current->keymenu      = &config_yesno_keymenu;
 	    current->help	  = config_help(vtmp - ps->vars, 0);
 	    current->tool	  = yesno_tool;
@@ -867,6 +879,7 @@ option_screen(ps)
 	    {int   rc, found = 0;
 	     char *result = NULL, buf[64];
 	     static char last[64];
+	     HelpType help;
 
 	     ps->mangled_footer = 1;
 	     buf[0] = '\0';
@@ -874,9 +887,12 @@ option_screen(ps)
 		     (last[0]) ? "[" : "",
 		     (last[0]) ? last : "",
 		     (last[0]) ? "]" : "");
+	     help = NO_HELP;
 	     while(1) {
-		 rc = optionally_enter(buf,-3,0,63,1,0,tmp,NULL,NO_HELP,0);
-		 if(rc == 0 || rc == 1 || !buf[0]){
+		 rc = optionally_enter(buf,-3,0,63,1,0,tmp,NULL,help,0);
+		 if(rc == 3)
+		   help = help == NO_HELP ? h_config_whereis : NO_HELP;
+		 else if(rc == 0 || rc == 1 || !buf[0]){
 		     if(rc == 0 && !buf[0] && last[0])
 		       strcpy(buf, last);
 
@@ -1030,7 +1046,7 @@ config_help(var, feature)
 	  case F_SELECT_WO_CONFIRM :
 	    return(h_config_select_wo_confirm);
 	  case F_COMPOSE_TO_NEWSGRP :
-	    return(h_config_compose_to_newsgrp_wo_confirm);
+	    return(h_config_compose_news_wo_conf);
 	  case F_USE_CURRENT_DIR :
 	    return(h_config_use_current_dir);
 	  case F_SAVE_WONT_DELETE :
@@ -1060,13 +1076,19 @@ config_help(var, feature)
 	  case F_SELECTED_SHOWN_BOLD :
 	    return(h_config_select_in_bold);
 	  case F_NO_NEWS_VALIDATION :
-	    return(h_config_post_without_validation);
+	    return(h_config_post_wo_validation);
 	  case F_ENABLE_INCOMING :
 	    return(h_config_enable_incoming);
 	  case F_ATTACHMENTS_IN_REPLY :
 	    return(h_config_attach_in_reply);
-	  case F_FORCE_USER_LOOKUP:
-	    return(h_config_force_user_lookup);
+	  case F_QUELL_LOCAL_LOOKUP :
+	    return(h_config_quell_local_lookup);
+	  case F_PRESERVE_START_STOP :
+	    return(h_config_preserve_start_stop);
+	  case F_COMPOSE_REJECTS_UNQUAL:
+	    return(h_config_compose_rejects_unqual);
+	  case F_FAKE_NEW_IN_NEWS:
+	    return(h_config_news_uses_recent);
 	  default :
 	    return(NO_HELP);
         }
@@ -1165,6 +1187,7 @@ text_tool(ps, cmd, cl, line)
     int		     rv = 0, i;
     struct variable *vtmp;
     CONF_S	    *ctmp;
+    HelpType         help;
 
     switch(cmd){
       case 'a' :				/* add to list */
@@ -1178,12 +1201,13 @@ text_tool(ps, cmd, cl, line)
 			    "\007Only single value allowed.  Use \"Change\".");
 	}
 	else{
-	    strcpy(prompt, "Enter text to add : ");
+	    strcpy(prompt, "Enter the value to be added : ");
 	    sval[0] = '\0';
 	    ps->mangled_footer = 1;
+	    help = NO_HELP;
 	    while(1){
 		i = optionally_enter(sval, -3, 0, MAXPATH, 1, 0, prompt,
-				     NULL, NO_HELP, 0);
+				     NULL, help, 0);
 		if(i == 0){
 		    rv = ps->mangled_body = 1;
 		    removing_leading_white_space(sval);
@@ -1226,7 +1250,8 @@ text_tool(ps, cmd, cl, line)
 		else if(i == 1){
 		    q_status_message(0,0,3,"Add cancelled");
 		}
-		else if(i == 3){		/* no help, yet */
+		else if(i == 3){
+		    help = help == NO_HELP ? h_config_add : NO_HELP;
 		    continue;
 		}
 		else if(i == 4){		/* no redraw, yet */
@@ -1241,16 +1266,25 @@ text_tool(ps, cmd, cl, line)
 
       case 'd' :				/* delete from list */
       case PF10 :
-	if((*cl)->var->is_fixed){
-	    q_status_message(0, 0, 3,
-			     "\007Can't delete sys-admin defined value.");
-	}
-	else if(((*cl)->var->is_list && !(*cl)->var->user_val.l)
+	if(((*cl)->var->is_list && !(*cl)->var->user_val.l)
 		|| (!(*cl)->var->is_list && !(*cl)->var->user_val.p)){
 	    q_status_message(0, 0, 3, "\007No set value to delete");
 	}
 	else{
-	    sprintf(prompt, "Really delete %s%.20s from %.30s ",
+	    if((*cl)->var->is_fixed)
+	        sprintf(prompt, "Delete (unused) %.30s from %.20s ",
+		    (*cl)->var->is_list
+		      ? (!*(*cl)->var->user_val.l[(*cl)->varmem])
+			  ? "<Empty Value>"
+			  : (*cl)->var->user_val.l[(*cl)->varmem]
+		      : ((*cl)->var->user_val.p)
+			  ? (!*(*cl)->var->user_val.p)
+			      ? "<Empty Value>"
+			      : (*cl)->var->user_val.p
+		 	  : "<NULL VALUE>",
+		    (*cl)->var->name);
+	    else
+	        sprintf(prompt, "Really delete %s%.20s from %.30s ",
 		    (*cl)->var->is_list ? "item " : "", 
 		    (*cl)->var->is_list
 		      ? int2string((*cl)->varmem + 1)
@@ -1293,6 +1327,8 @@ text_tool(ps, cmd, cl, line)
 			     "\007No current value set.  Use \"Add\".");
 	}
 	else{
+	    HelpType help;
+
 	    ps->mangled_footer = 1;
 	    if((*cl)->var->is_list){
 		sprintf(prompt, "Change field %.30s list entry : ",
@@ -1309,9 +1345,10 @@ text_tool(ps, cmd, cl, line)
 	    }
 
 	    ps->mangled_footer = 1;
+	    help = NO_HELP;
 	    while(1){
 		i = optionally_enter(sval, -3, 0, MAXPATH, 1, 0, prompt,
-				      NULL, NO_HELP, 0);
+				      NULL, help, 0);
 		if(i == 0){
 		    removing_leading_white_space(sval);
 		    removing_trailing_white_space(sval);
@@ -1366,7 +1403,8 @@ text_tool(ps, cmd, cl, line)
 		else if(i == 1){
 		    q_status_message(0,0,3,"Change cancelled");
 		}
-		else if(i == 3){		/* no help, yet */
+		else if(i == 3){
+		    help = help == NO_HELP ? h_config_change : NO_HELP;
 		    continue;
 		}
 		else if(i == 4){		/* no redraw, yet */
@@ -1406,7 +1444,7 @@ text_tool(ps, cmd, cl, line)
 	 * Now go and set the current_val based on user_val changes
 	 * above.  Turn off command line settings...
 	 */
-	set_current_val((*cl)->var, FALSE, FALSE);
+	set_current_val((*cl)->var, TRUE, FALSE);
 
 	/*
 	 * Handle any special case variable or config setting here
@@ -1698,6 +1736,22 @@ radiobutton_tool(ps, cmd, cl, line)
     int	       rv = 0;
     CONF_S    *ctmp;
 
+    if((*cl)->var->is_fixed){
+	q_status_message(0, 0, 3,
+			 "\007Can't change sys-admin defined value.");
+	if((*cl)->var->user_val.p){
+	    flush_status_messages();
+	    sleep(1);
+	    if(want_to("Delete old unused personal option setting", 'y', 'n',
+		        NO_HELP, 0, 1) == 'y'){
+		fs_give((void **)&(*cl)->var->user_val.p);
+		q_status_message(0, 0, 3, "Deleted");
+		rv = 1;
+	    }
+	}
+	return(rv);
+    }
+
     switch(cmd){
       case ctrl('M') :
       case ctrl('J') :
@@ -1743,8 +1797,9 @@ radiobutton_tool(ps, cmd, cl, line)
 	    rv = 1;
 	}
 	else if((*cl)->var == &ps->vars[V_SORT_KEY]){
-	    ps->def_sort_rev  = (*cl)->varmem >= EndofList;
-	    ps->def_sort      = (*cl)->varmem - (ps->def_sort_rev * EndofList);
+	    ps->def_sort_rev  = (*cl)->varmem >= (short) EndofList;
+	    ps->def_sort      = (SortOrder) ((*cl)->varmem - (ps->def_sort_rev
+								 * EndofList));
 	    if((*cl)->var->user_val.p)
 	      fs_give((void **)&(*cl)->var->user_val.p);
 
@@ -1782,7 +1837,7 @@ radiobutton_tool(ps, cmd, cl, line)
 #endif
 	else
 	  q_status_message(0, 0, 3,
-			   "\007Programmer botch!  Unknown checkbox type.");
+			   "\007Programmer botch!  Unknown radiobutton type.");
 
 	break;
 
@@ -1811,6 +1866,16 @@ yesno_tool(ps, cmd, cl, line)
     if((*cl)->var->is_fixed){
 	q_status_message(0, 0, 3,
 			 "\007Can't change sys-admin defined value.");
+	if((*cl)->var->user_val.p){
+	    flush_status_messages();
+	    sleep(1);
+	    if(want_to("Delete old unused personal option setting", 'y', 'n',
+		        NO_HELP, 0, 1) == 'y'){
+		fs_give((void **)&(*cl)->var->user_val.p);
+		q_status_message(0, 0, 3, "Deleted");
+		rv = 1;
+	    }
+	}
 	return(rv);
     }
 
@@ -1983,16 +2048,15 @@ pretty_value(ps, cl)
     char tmp[MAXPATH];
 
     if(cl->var->is_list){
-	if(cl->var->user_val.l){
+	if(!cl->var->is_fixed && cl->var->user_val.l){
 	    sprintf(tmp, "%-*s", ps->ttyo->screen_cols - cl->valoffset,
 		    (*cl->var->user_val.l[cl->varmem])
 		       ? cl->var->user_val.l[cl->varmem]
-		       : "<Empty Value>",
-		    ps->ttyo->screen_cols - cl->valoffset);
+		       : "<Empty Value>");
 	}
 	else{
 	    char *p = tmp;
-	    sstrcpy(&p, "<No Value Set");
+	    sstrcpy(&p, cl->var->is_fixed?"<Value is Fixed":"<No Value Set");
 	    if(cl->var->current_val.l){
 		int i, l, l2;
 
@@ -2027,8 +2091,10 @@ pretty_value(ps, cl)
 	}
 
     }
-    else if(!cl->var->user_val.p){
-	sprintf(tmp, "<No Value Set%s%s%s>%*s", 
+    else if(cl->var->is_fixed || !cl->var->user_val.p){
+	sprintf(tmp, cl->var->is_fixed
+			? "<Value is Fixed%s%s%s>%*s" :
+			"<No Value Set%s%s%s>%*s", 
 		(cl->var->current_val.p) ? ": using \"" : "",
 		(cl->var->current_val.p) ? cl->var->current_val.p : "",
 		(cl->var->current_val.p) ? "\"" : "",
@@ -2184,6 +2250,11 @@ toggle_feature_bit(ps, index, current)
      */
     if(f->value == F_QUOTE_ALL_FROMS)
       mail_parameters(NULL,SET_FROMWIDGET,(void *)(F_ON(f->value,ps) ? 1 : 0));
+    else if(f->value == F_PRESERVE_START_STOP){
+	/* toggle raw mode settings to make tty driver aware of new setting */
+	Raw(0);
+	Raw(1);
+    }
 }
 
 
@@ -2252,3 +2323,140 @@ first_confline(p)
 
     return(p);
 }
+
+
+int
+offer_to_fix_pinerc(ps)
+    struct pine *ps;
+{
+    struct variable *v;
+    char             prompt[300];
+    char            *p, *q;
+    char           **list;
+    char           **list_fixed;
+    int              rv = 0;
+    int              i, k, need;
+    char            *clear = ": clear";
+
+    ps->fix_fixed_warning = 0;  /* so we only ask first time */
+
+    if(want_to("Some of your options conflict with site policy.  Investigate",
+	'y', 'n', NO_HELP, 0, 1) != 'y')
+      return(rv);
+    
+/* space want_to requires in addition to the string you pass in */
+#define WANTTO_SPACE 6
+    need = WANTTO_SPACE + strlen(clear);
+
+    for(v = ps->vars; v->name; v++){
+	if(!v->is_fixed ||
+	   !v->is_user ||
+	    v->is_obsolete ||
+	    v == &ps->vars[V_FEATURE_LIST]) /* handle feature-list below */
+	  continue;
+	
+	prompt[0] = '\0';
+	
+	if(v->is_list && v->user_val.l){
+	    if(*v->user_val.l){
+		sprintf(prompt, "Your setting for %s is ", v->name);
+		p = prompt + strlen(prompt);
+		for(i = 0; v->user_val.l[i]; i++){
+		    if(p - prompt > ps->ttyo->screen_cols - need)
+		      break;
+		    if(i)
+		      *p++ = ',';
+		    sstrcpy(&p, v->user_val.l[i]);
+		}
+		*p = '\0';
+	    }
+	    else
+	      sprintf(prompt, "Your setting for %s is <Empty Value>", v->name);
+	}
+	else{
+	    if(v->user_val.p){
+		if(*v->user_val.p){
+		    sprintf(prompt, "Your setting for %s is %s",
+			v->name, v->user_val.p);
+		}
+		else{
+		    sprintf(prompt, "Your setting for %s is <Empty Value>",
+			v->name);
+		}
+	    }
+	}
+	if(*prompt){
+	    if(strlen(prompt) > ps->ttyo->screen_cols - need)
+	      (void)strcpy(prompt + max(ps->ttyo->screen_cols - need - 3, 0),
+			  "...");
+
+	    (void)strcat(prompt, clear);
+	    if(want_to(prompt, 'y', 'n', NO_HELP, 0, 0) == 'y'){
+		if(v->is_list){
+		    if(v->user_val.l){
+			rv++;
+			for(i = 0; v->user_val.l[i]; i++)
+			  fs_give((void **)&v->user_val.l[i]);
+
+			fs_give((void **)&v->user_val.l);
+		    }
+		}
+		else if(v->user_val.p){
+		    rv++;
+		    fs_give((void **)&v->user_val.p);
+		}
+	    }
+	}
+    }
+
+    /*
+     * As always, feature-list has to be handled separately.
+     */
+    v = &ps->vars[V_FEATURE_LIST];
+    list = v->user_val.l;
+    list_fixed = v->fixed_val.l;
+    if(list){
+      for(i = 0; list[i]; i++){
+	p = list[i];
+	if(!struncmp(p, "no-", 3))
+	  p += 3;
+	for(k = 0; list_fixed && list_fixed[k]; k++){
+	  q = list_fixed[k];
+	  if(!struncmp(q, "no-", 3))
+	    q += 3;
+	  if(!strucmp(q, p)){
+	    sprintf(prompt, "Your %s is %s, fixed value is %s",
+		p, p == list[i] ? "ON" : "OFF",
+		q == list_fixed[k] ? "ON" : "OFF");
+
+	    if(strlen(prompt) > ps->ttyo->screen_cols - need)
+	      (void)strcpy(prompt + max(ps->ttyo->screen_cols - need - 3, 0),
+			  "...");
+
+	    (void)strcat(prompt, clear);
+	    if(want_to(prompt, 'y', 'n', NO_HELP, 0, 0) == 'y'){
+		rv++;
+		/*
+		 * Clear the feature from the user's pinerc
+		 * so that we'll stop bothering them when they
+		 * start up Pine.
+		 */
+		clear_feature(&v->user_val.l, p);
+
+		/*
+		 * clear_feature scoots the list up, so if list[i] was
+		 * the last one going in, now it is the end marker.  We
+		 * just decrement i so that it will get incremented and
+		 * then test == 0 in the for loop.  We could just goto
+		 * outta_here to accomplish the same thing.
+		 */
+		if(!list[i])
+		  i--;
+	    }
+	  }
+	}
+      }
+    }
+
+    return(rv);
+}
