@@ -1,5 +1,5 @@
 #if !defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: pine.c,v 4.207 1994/08/25 22:40:50 mikes Exp $";
+static char rcsid[] = "$Id: pine.c,v 4.225 1994/10/11 00:27:14 mikes Exp $";
 #endif
 /*----------------------------------------------------------------------
 
@@ -267,7 +267,7 @@ main(argc, argv)
       fix the tty driver to always complete the write.
      */
     setbuffer(stdout, stdiobuf, 64);
-#endif	
+#endif
 
     /* need home directory early */
     get_user_info(&ps_global->ui);
@@ -317,17 +317,27 @@ main(argc, argv)
      * install c-client callback to manage cache data outside
      * free memory
      */
-    mailcache = dos_cache;
+    mail_parameters(NULL, SET_CACHE, (void *)dos_cache);
 
     /*
-     * Set up a c-client read timeout and timeout handler.  tunger
-     * reports periodic blocking on read when trying to talk to a 
-     * dead imapd under winsock.  It shouldn't happen, but we'll set
-     * up handlers just in case.  Since we're here, we might as well
-     * make it available to DOS as well...
+     * Set up a c-client read timeout and timeout handler.  In general,
+     * it shouldn't happend, but a server crash or dead link can cause
+     * pine to appear wedged if we don't set this up...
      */
     mail_parameters(NULL, SET_READTIMEOUT, (void *)15L);
-    tcptimeout = dos_tcptimeout;
+    mail_parameters(NULL, SET_TIMEOUT, (void *)dos_tcptimeout);
+
+    /*
+     * Sniff the environment for timezone offset.  We need to do this
+     * here since Windows needs help figuring out UTC, and will adjust
+     * what time() returns based on TZ.  THIS WILL SCREW US because
+     * we use time() differences to manage status messages.  So, if 
+     * rfc822_date, which calls localtime() and thus needs tzset(),
+     * is called while a status message is displayed, it's possible
+     * for time() to return a time *before* what we remember as the
+     * time we put the status message on the display.  Sheesh.
+     */
+    tzset();
 #endif
 
     /*------- Set up c-client drivers -------------- */ 
@@ -344,8 +354,6 @@ main(argc, argv)
     if(!pine_state->nr_mode)
       write_pinerc(pine_state);
 
-    mailcap_init();		/* Read and process mailcap */
-
     if(!pine_state->more_mode && *argv == NULL)
       /*
        * do mail dir if we're not in send only mode...
@@ -663,6 +671,10 @@ main(argc, argv)
 	pine_state->in_init_seq = pine_state->save_in_init_seq;
 	pine_state->dont_use_init_cmds = 0;
 
+	if(pine_state->give_fixed_warning)
+	    q_status_message(0, 2, 10,
+"Note: some of your config options conflict with site policy and are ignored");
+
         /*-------------------------------------------------------------------
                          Loop executing the commands
     
@@ -679,6 +691,11 @@ main(argc, argv)
             if(pine_state->next_screen == SCREEN_FUN_NULL) 
               pine_state->next_screen = main_menu_screen;
 
+#ifdef	_WINDOWS
+	    /* make sure each screen sets exit char and it's gray otherwise */
+	    mswin_allowexit (MSWIN_EXIT_DENY, NULL, '\0');
+#endif
+
             (*(pine_state->next_screen))(pine_state);
         }
     }
@@ -815,6 +832,8 @@ main_menu_screen(pine_state)
     ch                        = 'x'; /* For display_message 1st time through */
     pine_state->prev_screen   = main_menu_screen;
 
+    mailcap_free(); /* free resources we won't be using for a while */
+
     if(!pine_state->painted_body_on_startup 
        && !pine_state->painted_footer_on_startup){
 	ClearScreen();
@@ -865,6 +884,10 @@ main_menu_screen(pine_state)
          * will have to change under UNIX if we want to do the same.
 	 */
 	while_waiting = build_header_cache;
+#ifdef	_WINDOWS
+	mswin_allowexit (MSWIN_EXIT_SENDCHAR, NULL,
+			 F_ON(F_USE_FK, ps_global) ? OPF3 : 'q');
+#endif
 #endif
         ch = read_command();
 #ifdef	DOS
@@ -875,6 +898,9 @@ main_menu_screen(pine_state)
 	    clear_mfunc();
 	}
 #endif
+#ifdef	_WINDOWS
+	mswin_allowexit (MSWIN_EXIT_DENY, "Complete current operation", '\0');
+#endif
 #endif
         orig_ch = ch;
 
@@ -1383,7 +1409,7 @@ setup_mini_menu(ql)
 
     setup_names[ekey_num].ch    = -1;
 
-    s = radio_buttons(prompt, ql, 0, setup_names, deefault, 'x', 0, help);
+    s = radio_buttons(prompt, ql, 0, setup_names, deefault, 'x', 0, help, 0);
     /* ^C */
     if(s == 'x') {
 	q_status_message(0,0,3,"Setup command cancelled");
@@ -1431,12 +1457,7 @@ do_setup_task(command)
 
         /*----- NEW PASSWORD -----*/
       case 'n':
-#ifdef	DOS
-        q_status_message(1, 3, 5,
-	    "Password change not available in the DOS version of Pine.");
-	display_message('x');
-        sleep(3);
-#else
+#ifdef	PASSWD_PROG
         if(ps_global->restricted){
 	    q_status_message(1, 3, 5,
 	    "Password change unavailable in restricted demo version of Pine.");
@@ -1447,6 +1468,10 @@ do_setup_task(command)
 	    ClearScreen();
 	    ps_global->mangled_screen = 1;
 	}
+#else
+        q_status_message(0, 3, 5,
+		 "Password changing not configured for this version of Pine.");
+	display_message('x');
 #endif	/* DOS */
         break;
 
@@ -1706,14 +1731,15 @@ new_user_or_version(message)
 		 "PINE is a trademark of the University of Washington.");
 
     /*
-     * You may think this is weird.  We're trying to only offer sending
+     * You may think this is weird.  We're trying to offer sending
      * the "phone home" message iff there's no evidence we've ever
-     * run pine *or* this is the most recent version we've *ever* run.
-     * The check for the existance of a var new in 3.90, is to compensate
-     * for pre 3.90 pine's rewriting their lower version number.
+     * run pine *or* this is the first time we've run a version
+     * a version of pine >= 3.90.  The check for the existence of a var
+     * new in 3.90, is to compensate for pre 3.90 pine's rewriting their
+     * lower version number.
      */
-    if(ps_global->first_time_user || !ps_global->pre390
-       || !var_in_pinerc(ps_global->vars[V_NNTP_SERVER].name)){
+    if(ps_global->first_time_user || (ps_global->pre390
+       && !var_in_pinerc(ps_global->vars[V_NNTP_SERVER].name))){
 	phone_home_blurb(dline);
 	if(want_to("Request document", 'y', 0, NO_HELP, 0, 1) == 'y')
 	  phone_home();
@@ -1828,7 +1854,7 @@ phone_home_blurb(dline)
 {
     static char *blurb[] = {
       "SPECIAL OFFER:  Would you like to receive (via email) a brief document",
-      "  entitled \"Secrets of Pine 3.90\" ?"
+      "  entitled \"Secrets of Pine\" ?"
     };
 
     if(dline+3 < ps_global->ttyo->screen_rows - 3)
@@ -1923,10 +1949,12 @@ quit_screen(pine_state)
     end_screen();
     end_keyboard(F_ON(F_USE_FK,pine_state));
     end_tty_driver(pine_state);
+    end_signals(0);
 
     imap_flush_passwd_cache();
     clear_index_cache();
     completely_done_with_adrbks();
+    mailcap_free();
     free_folders();
 
     if(pine_state->hostname != NULL)
@@ -1951,7 +1979,8 @@ quit_screen(pine_state)
     fs_give((void **)&pine_state);
 
 #ifdef DEBUG
-    fclose(debugfile);
+    if(debugfile)
+      fclose(debugfile);
 #endif    
 
     printf("\n\n\nPine finished\n\n");
@@ -1963,22 +1992,26 @@ quit_screen(pine_state)
 /*
  * Useful flag checking macro for 
  */
-#define FLAG_MATCH(F, M)   (((((F)&F_SEEN) ? (M)->seen			     \
-				: ((F)&F_UNSEEN) ? !(M)->seen : 1)	     \
-			  && (((F)&F_DEL)  ? (M)->deleted		     \
-				: ((F)&F_UNDEL)  ? !(M)->deleted : 1)	     \
+#define FLAG_MATCH(F, M)   (((((F)&F_SEEN)   ? (M)->seen		     \
+				: ((F)&F_UNSEEN)     ? !(M)->seen : 1)	     \
+			  && (((F)&F_DEL)    ? (M)->deleted		     \
+				: ((F)&F_UNDEL)      ? !(M)->deleted : 1)    \
+			  && (((F)&F_ANS)    ? (M)->answered		     \
+				: ((F)&F_UNANS)	     ? !(M)->answered : 1)   \
 			  && (((F)&F_FLAG) ? (M)->flagged		     \
-				: ((F)&F_UNFLAG) ? !(M)->flagged : 1)	     \
-			  && (((F)&F_ANS)  ? (M)->answered		     \
-				: ((F)&F_ANS)    ? !(M)->answered : 1))	     \
+				: ((F)&F_UNFLAG)   ? !(M)->flagged : 1)	     \
+			  && (((F)&F_RECENT) ? (M)->recent		     \
+				: ((F)&F_UNRECENT)   ? !(M)->recent : 1))    \
 			  || ((((F)&F_OR_SEEN) ? (M)->seen		     \
-				: ((F)&F_OR_UNSEEN) ? !(M)->seen : 0)        \
+				: ((F)&F_OR_UNSEEN)   ? !(M)->seen : 0)      \
 			  || (((F)&F_OR_DEL)   ? (M)->deleted		     \
-				: ((F)&F_OR_UNDEL)  ? !(M)->deleted : 0)     \
-			  || (((F)&F_OR_FLAG)  ? (M)->flagged		     \
-				: ((F)&F_OR_UNFLAG) ? !(M)->flagged : 0)     \
+				: ((F)&F_OR_UNDEL)    ? !(M)->deleted : 0)   \
 			  || (((F)&F_OR_ANS)   ? (M)->answered		     \
-				: ((F)&F_OR_ANS)    ? !(M)->answered : 0)))
+				: ((F)&F_OR_ANS)      ? !(M)->answered : 0)  \
+			  || (((F)&F_OR_FLAG)? (M)->flagged		     \
+				: ((F)&F_OR_UNFLAG) ? !(M)->flagged : 0)     \
+			  || (((F)&F_OR_RECENT)? (M)->recent		     \
+				: ((F)&F_OR_UNRECENT) ? !(M)->recent : 0)))
 
 
 
@@ -1992,23 +2025,20 @@ quit_screen(pine_state)
   ----------------------------------------------------------------------*/
 MsgNo
 first_sorted_flagged(flags, stream)
-    unsigned int  flags;
-    MAILSTREAM   *stream;
+    unsigned long  flags;
+    MAILSTREAM    *stream;
 {
     MsgNo        i;
     MESSAGECACHE *mc;
 
     FETCH_ALL_FLAGS(stream);
 
-    for(i = 1 ; i <= mn_get_total(ps_global->msgmap); i++) {
+    for(i = 1 ; i < mn_get_total(ps_global->msgmap); i++) {
 	mc = mail_elt(stream, mn_m2raw(ps_global->msgmap, i));
 	if(mc && FLAG_MATCH(flags, mc))
           break;
     }
 
-    if(i > stream->nmsgs)
-      i = stream->nmsgs;
-
     dprint(4, (debugfile, "First unseen returning %ld\n", (long)i));
     return(i);
 }
@@ -2029,10 +2059,10 @@ first_sorted_flagged(flags, stream)
   ----------------------------------------------------------------------*/
 MsgNo
 next_sorted_flagged(flags, stream, start, found_is_new)
-    unsigned int  flags;
-    MAILSTREAM   *stream;
-    long          start;
-    int          *found_is_new;
+    unsigned long  flags;
+    MAILSTREAM    *stream;
+    long           start;
+    int           *found_is_new;
 {
     MsgNo        i;
     MESSAGECACHE *mc;
@@ -2207,18 +2237,20 @@ count_flagged(stream, flags)
      MAILSTREAM *stream;
      char       *flags;
 {
-    extern MAILSTREAM *mm_search_stream;
-    extern MSGNO_S    *mm_search_map, *mm_exclude_map;
-    extern int	       mm_search_count;
-    int                old_prefetch;
+    char   *flagp;
+    int     old_prefetch;
+    extern  MAILSTREAM *mm_search_stream;
+    extern  MSGNO_S    *mm_search_map, *mm_exclude_map;
+    extern  int	       mm_search_count;
 
     mm_search_stream = stream;
     mm_search_map    = mm_exclude_map = NULL;
     mm_search_count  = 0L;
 
     old_prefetch = (int) mail_parameters(stream, SET_PREFETCH, NULL);
-    mail_search(stream, flags);
+    mail_search(stream, flagp = cpystr(flags));
     mail_parameters(stream, SET_PREFETCH, (void *) old_prefetch);
+    fs_give((void **)&flagp);
 
     return(mm_search_count);
 }
@@ -2355,13 +2387,13 @@ panic(message)
     end_screen();
     end_keyboard(ps_global != NULL ? F_ON(F_USE_FK,ps_global) : 0);
     end_tty_driver(ps_global);
-    end_signals();
+    end_signals(1);
     dprint(1, (debugfile, "Pine Panic: %s\n", message));
     fprintf(stderr, "\n\nBug in Pine detected: \"%s\".\nExiting pine.\n",
             message);
 #ifdef DEBUG
-    if(debug)
-      fclose(debugfile);
+    if(debugfile)
+      save_debug_on_crash(debugfile);
     coredump();   /*--- If we're debugging get a core dump --*/
 #endif
 
