@@ -1,5 +1,5 @@
 #if !defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: print.c,v 4.19 1994/07/26 17:46:21 mikes Exp $";
+static char rcsid[] = "$Id: print.c,v 4.21 1994/10/05 18:37:55 mikes Exp $";
 #endif
 /*----------------------------------------------------------------------
 
@@ -125,7 +125,7 @@ open_printer(desc)
 	write_error = 1;	/* just in case */
         return(-1);
     }
-    q_status_message(1, 2, 3,"Printing to windows printer...");
+    q_status_message(1, 0, 3,"Printing to windows printer...");
     display_message('x');
 #endif /* _WINDOWS */
 
@@ -134,7 +134,8 @@ open_printer(desc)
     int  p[2];
 
     if(ps_global->VAR_PRINTER == NULL) {
-        q_status_message(1,1,3,"\007No printer has been chosen.  Use SETUP on main menu to make choice.");
+        q_status_message(1, 1, 3,
+    "\007No printer has been chosen.  Use SETUP on main menu to make choice.");
         return(-1);
     }
     outputsave = fs_get(40) ;
@@ -148,7 +149,7 @@ open_printer(desc)
         sprintf(prompt, "Print %susing \"%s\"",desc == NULL ? "" : desc,
                 ps_global->VAR_PRINTER);
         if(want_to(prompt, 'y', 'n', (char **)NULL, 0, 0) == 'n') {
-            q_status_message(0, 0,2, "Print cancelled");
+            q_status_message(0, 0, 2, "Print cancelled");
             return(-1);
         }
     } else {
@@ -157,11 +158,11 @@ open_printer(desc)
         sprintf(prompt, "Print %susing command \"%s\"", desc==NULL ? "" : desc,
                 ps_global->VAR_PRINTER);
         if(want_to(prompt, 'y', 'n', (char **)NULL, 0, 0) == 'n') {
-            q_status_message(0, 0,2, "Print cancelled");
+            q_status_message(0, 0, 2, "Print cancelled");
             return(-1);
         }
         strcpy(command, ps_global->VAR_PRINTER);
-        q_status_message1(1, 1,3,"Printing with command \"%s\"",
+        q_status_message1(1, 1, 3, "Printing with command \"%s\"",
                           ps_global->VAR_PRINTER);
     }
 
@@ -173,10 +174,9 @@ open_printer(desc)
         display_message('x');
         sleep(1);
         Raw(0);
-        printf("\033[5i");
+        fputs("\033[5i", stdout);
         command_pipe = stdout;
-
-    }  else {
+    } else {
         /*----------- Print by forking off a UNIX command ------------*/
 	print_pipe = open_system_pipe(ps_global->VAR_PRINTER, 
 				      &outputsave, PIPE_SYS) ;
@@ -215,20 +215,20 @@ void close_printer()
 #else
     mswin_print_done ();
 #endif
-    q_status_message(0, 1, 3, "Printing complete");
 #else
     int   status ;
   
     if(command_pipe == stdout) {
-        printf("\f\033[4i");
+        fputs("\f\033[4i", stdout);
         fflush(stdout);
         Raw(1);
     } else {
-	status = close_system_pipe(&print_pipe);
+	status = close_system_pipe(&print_pipe, PIPE_SYS);
 	display_system_pipe_output(outputsave, "PRINT") ;
     }
 #endif	/* !DOS */
-    
+
+    q_status_message(0, 0, 3, "Print command completed");
     display_message('x');
 }
 
