@@ -1,5 +1,5 @@
 #if !defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: reply.c,v 4.84 1994/08/19 01:03:27 mikes Exp $";
+static char rcsid[] = "$Id: reply.c,v 4.94 1994/10/06 22:13:29 mikes Exp $";
 #endif
 /*----------------------------------------------------------------------
 
@@ -108,6 +108,7 @@ char	*forward_subject();
 #define	FRM_PMT	"Use \"Reply to:\" address instead of \"From:\" address"
 #define	ALL_PMT		"Reply to all recipients"
 #define	NEWS_PMT	"Post follow-up message to news group(s)"
+#define	BUG_PMT		"Attach current message to report"
 
 /*
  * standard type of storage object used for body parts...
@@ -189,7 +190,8 @@ reply(pine_state)
     cc_tail		  = &outgoing->cc;
     outgoing->to	  = (ADDRESS *)NULL;
     outgoing->cc	  = (ADDRESS *)NULL;
-    outgoing->subject = NULL;
+    outgoing->subject	  = NULL;
+    ret			  = 0;
     for(msgno = mn_first_cur(pine_state->msgmap);
 	msgno > 0L;
 	msgno = mn_next_cur(pine_state->msgmap)){
@@ -205,8 +207,15 @@ reply(pine_state)
 	    goto done_early;
 	}
 
-	if((totalm == 1L || IS_NEWS(pine_state->mail_stream))
-	   && env->newsgroups && env->newsgroups[0] && !outgoing->newsgroups
+	/*
+	 * If we haven't asked before, and we're replying to one
+	 * messsage (an agg reply in a non-news folder may not have
+	 * newsgroups in all the members or even different news groups)
+	 * or an agg reply in a news folder, and the envelope has a
+	 * newsgroup, ask if the user is posting a follow-up article.
+	 */
+	if(!ret && (totalm == 1L || IS_NEWS(pine_state->mail_stream))
+	   && env->newsgroups && env->newsgroups[0]
 	   && (ret = want_to(NEWS_PMT, 'n', 'x', NO_HELP, 0, 0)) == 'y'){
 	    /*========= Follow up to a news group ============*/
 	    outgoing->newsgroups = cpystr(env->newsgroups);
@@ -221,7 +230,8 @@ reply(pine_state)
 
 /*
  * BUG: we need to decide what to do if reply in mixed news/mail folder
- * and some message hve newsgroups and some don't
+ * and some message have newsgroups and some don't or the ones that do
+ * don't all match.
  */
 
 	    /*---- Set up the References: field ----*/
@@ -381,6 +391,13 @@ reply(pine_state)
 
     outgoing->message_id  = generate_message_id(pine_state);
 
+    if(!outgoing->to &&
+       !outgoing->cc &&
+       !outgoing->bcc &&
+       !outgoing->newsgroups)
+      q_status_message(0, 4, 6,
+		       "\007Warning: no valid addresses to reply to!");
+
 #if	defined(DOS) && !defined(_WINDOWS)
 #if	defined(LWP) || defined(PCTCP) || defined(PCNFS)
 #define	IN_RESERVE	8192
@@ -583,7 +600,8 @@ reply(pine_state)
 		if(part->body.contents.binary = (void *) so_get(PART_SO_TYPE,
 							    NULL,EDIT_ACCESS)){
 #ifdef	DOS
-		    mailgets    = dos_gets;	/* fetched text to disk */
+		    mail_parameters(ps_global->mail_stream, SET_GETS,
+				    (void *)dos_gets); /* fetched to disk */
 		    append_file = (FILE *)so_text(
 					(STORE_S *)part->body.contents.binary);
 
@@ -591,7 +609,8 @@ reply(pine_state)
 				      &part->body.size.bytes) == NULL)
 		      goto done;
 
-		    mailgets    = NULL;
+		    mail_parameters(ps_global->mail_stream, SET_GETS,
+				    (void *)NULL);
 		    append_file = NULL;
 		    mail_gc(pine_state->mail_stream, GC_TEXTS);
 #else
@@ -620,7 +639,7 @@ reply(pine_state)
 #endif
 
     /* partially formatted outgoing message */
-    pine_send(outgoing, &body, "COMPOSE MESSAGE REPLY", fcc, seq, refs, NULL);
+    pine_send(outgoing, &body, "COMPOSE MESSAGE REPLY", fcc, seq, refs, NULL,0);
 
   done:
     pine_free_body(&body);
@@ -1052,7 +1071,8 @@ forward(pine_state)
 	      goto bomb;
 
 #ifdef	DOS
-	    mailgets    = dos_gets;	/* write fetched text to disk */
+	    /* write fetched text to disk */
+	    mail_parameters(stream, SET_GETS, (void *)dos_gets);
 	    append_file = (FILE *)so_text((STORE_S *)b->contents.binary);
 
 	    /* HACK!  See mailview.c:format_message for details... */
@@ -1063,7 +1083,7 @@ forward(pine_state)
 
 	    b->size.bytes = ftell(append_file);
 	    /* next time body may stay in core */
-	    mailgets      = NULL;
+	    mail_parameters(stream, SET_GETS, (void *)NULL);
 	    append_file   = NULL;
 	    mail_gc(stream, GC_TEXTS);
 	    so_release((STORE_S *)b->contents.binary);
@@ -1231,7 +1251,8 @@ forward(pine_state)
 	    if(part->body.contents.binary = (void *) so_get(PART_SO_TYPE, NULL,
 							    EDIT_ACCESS)){
 #ifdef	DOS
-		mailgets    = dos_gets;	/* fetched text to disk */
+		/* fetched text to disk */
+		mail_parameters(stream, SET_GETS, (void *)dos_gets);
 		append_file = (FILE *)so_text(
 				       (STORE_S *)part->body.contents.binary);
 
@@ -1240,7 +1261,7 @@ forward(pine_state)
 		  goto bomb;
 
 		/* next time body may stay in core */
-		mailgets    = NULL;
+		mail_parameters(stream, SET_GETS, (void *)NULL);
 		append_file = NULL;
 		mail_gc(stream, GC_TEXTS);
 #else
@@ -1264,7 +1285,7 @@ forward(pine_state)
     else			/* partially formatted outgoing message */
       pine_send(outgoing, &body,
 		pine_state->nr_mode ? "SEND MESSAGE" : "FORWARD MESSAGE",
-		NULL, NULL, NULL, NULL);
+		NULL, NULL, NULL, NULL, 0);
 
   clean:
     pine_free_body(&body);
@@ -1274,7 +1295,7 @@ forward(pine_state)
 
   bomb:
 #ifdef	DOS
-    mailgets    = NULL;
+    mail_parameters(stream, SET_GETS, (void *)NULL);
     append_file = NULL;
     mail_gc(pine_state->mail_stream, GC_TEXTS);
 #endif
@@ -1373,7 +1394,7 @@ forward_text(pine_state, text, source)
 		pine_state->mangled_footer = 1;
 	    }
 	    else{
-		pine_send(env, &body, "SEND MESSAGE", NULL, NULL, NULL, NULL);
+		pine_send(env, &body, "SEND MESSAGE",NULL,NULL,NULL,NULL,0);
 		pine_state->mangled_screen = 1;
 	    }
 	}
@@ -1416,11 +1437,13 @@ bounce(pine_state)
     MAILSTREAM    *stream;
     void          *msgtext;
     gf_io_t        pc;
-    char          *sig;
+    char          *sig, *fields[2];
     int            ret=0;
 
     dprint(4, (debugfile, "\n - bounce -\n"));
 
+    stream = pine_state->mail_stream;
+
     /*---------- Get the envelope of message we're replying to ------*/
     if(mn_total_cur(pine_state->msgmap) > 1L){
 	q_status_message1(1,2,4,
@@ -1430,7 +1453,7 @@ bounce(pine_state)
     }
     else{
 	msgno = mn_m2raw(pine_state->msgmap, mn_get_cur(pine_state->msgmap));
-	env   = mail_fetchstructure(pine_state->mail_stream, msgno, &body);
+	env   = mail_fetchstructure(stream, msgno, &body);
 	if(!env || !body) {
 	    q_status_message1(1,2,4,
 			      "Error fetching message %s. Can't reply to it.",
@@ -1439,18 +1462,18 @@ bounce(pine_state)
 	}
     }
 
-    stream                = pine_state->mail_stream;
     outgoing              = mail_newenvelope();
     outgoing->message_id  = generate_message_id(pine_state);
 
 
     /*==================== Subject ====================*/
-    if(env->subject == NULL || strlen(env->subject) == 0) {
-	/* --- Blank, make up one ---*/
-	outgoing->subject = cpystr("Resent mail....");
-      }
+    if(env->subject == NULL || strlen(env->subject) == 0)
+      /* --- Blank, make up one ---*/
+      outgoing->subject = cpystr("Resent mail....");
 
-    outgoing->remail = cpystr(mail_fetchheader(stream, msgno)) ;
+    fields[0] = "Received";
+    fields[1] = NULL;
+    outgoing->remail = xmail_fetchheader_lines_not(stream, msgno, fields);
 
     /*
      * as with all text bound for the composer, build it in 
@@ -1461,15 +1484,28 @@ bounce(pine_state)
 	goto clean_early;
     }
 
-    orig_body = body;
+    /*
+     * Build a fake body description.  It's ignored by pine_rfc822_header,
+     * but we need to set it to something that makes set_mime_types
+     * not sniff it and pine_rfc822_output_body not re-encode it.
+     * Setting the encoding to (ENCMAX + 1) will work and shouldn't cause
+     * problems unless something tries to access body_encodings[] using
+     * it without proper precautions.  We don't want to use ENCOTHER
+     * cause that tells set_mime_types to sniff it, and we don't want to
+     * use ENC8BIT since that tells pine_rfc822_output_body to qp-encode
+     * it.  When there's time, it'd be nice to clean this interaction
+     * up...
+     */
+    orig_body		  = body;
+    body		  = mail_newbody();
+    body->type		  = TYPETEXT;
+    body->encoding	  = ENCMAX + 1;
+    body->subtype	  = cpystr("Plain");
+    body->contents.binary = msgtext ;
     gf_set_so_writec(&pc, (STORE_S *)msgtext);
 
-    body = mail_newbody() ;
-    body->type = TYPETEXT ;
-    body->contents.binary = msgtext ;
-    get_body_part_text(pine_state->mail_stream, NULL, msgno, "1", pc, 
-		       "", ". Text not included");
-    
+    /* pass NULL body to force mail_fetchtext */
+    get_body_part_text(stream, NULL, msgno, "1", pc, "",". Text not included");
     pine_simple_send(outgoing, &body);
 
   clean:
@@ -1480,12 +1516,12 @@ bounce(pine_state)
 
   bomb:
 #ifdef	DOS
-    mailgets    = NULL;
+    mail_parameters(stream, SET_GETS, (void *)NULL);
     append_file = NULL;
     mail_gc(pine_state->mail_stream, GC_TEXTS);
 #endif
     q_status_message(1, 4, 5,
-		     "\007Error fetching message contents. Can't resend message");
+		  "\007Error fetching message contents. Can't resend message");
     goto clean;
 
 }
@@ -1509,6 +1545,7 @@ gripe(ps)
     PART     **pp;
     gf_io_t    pc;
     char       tmp[MAX_ADDRESS], *p, *sig;
+    int	       i, ch = 0;
     long       fake_reply = -1L,
 	       msgno = mn_m2raw(ps->msgmap, mn_get_cur(ps->msgmap));
 
@@ -1527,7 +1564,16 @@ gripe(ps)
     outgoing   = mail_newenvelope();
     rfc822_parse_adrlist(&outgoing->to, tmp, ps->maildomain);
     outgoing->message_id = generate_message_id(ps);
-    outgoing->subject    = cpystr("Bug Report: ");
+    /* Build our contribution to the subject; part constant string
+     * and part random 4 character alpha numeric string
+     */
+    sprintf(tmp_20k_buf, "Bug (ID %c%c%d%c%c): ", 
+	    ((i = (int)(random() % 36L)) < 10) ? '0' + i : 'A' + (i - 10),
+	    ((i = (int)(random() % 36L)) < 10) ? '0' + i : 'A' + (i - 10),
+	    (int)(random() % 10L),
+	    ((i = (int)(random() % 36L)) < 10) ? '0' + i : 'A' + (i - 10),
+	    ((i = (int)(random() % 36L)) < 10) ? '0' + i : 'A' + (i - 10));
+    outgoing->subject = cpystr(tmp_20k_buf);
 
     body       = mail_newbody();
     body->type = TYPEMULTIPART;
@@ -1565,6 +1611,7 @@ gripe(ps)
 
     if(pb->contents.binary = (void *) so_get(CharStar, NULL, EDIT_ACCESS)){
 	gf_set_so_writec(&pc, (STORE_S *)pb->contents.binary);
+	dump_pine_struct(ps, pc);
 	dump_config(ps, pc);
 	pb->size.bytes = strlen((char *)so_text(
 					      (STORE_S *)pb->contents.binary));
@@ -1574,8 +1621,8 @@ gripe(ps)
 	goto bomb;
     }
 
-    switch(want_to("Attach current message to report",'y','x',NO_HELP,0,0)){
-      case 'y' :
+    if(mn_get_total(ps->msgmap) > 0L
+       && (ch = one_try_want_to(BUG_PMT,'y','x',NO_HELP,0,1)) == 'y'){
 	*pp		      = mail_newbody_part();
 	pb		      = &((*pp)->body);
 	pb->type	      = TYPEMESSAGE;
@@ -1599,7 +1646,8 @@ gripe(ps)
 	  goto bomb;
 
 #ifdef	DOS
-	mailgets    = dos_gets;	/* write fetched text to disk */
+	/* write fetched text to disk */
+	mail_parameters(ps->mail_stream, SET_GETS, (void *)dos_gets);
 	append_file = (FILE *)so_text((STORE_S *)pb->contents.binary);
 
 	/* HACK!  See mailview.c:format_message for details... */
@@ -1610,7 +1658,7 @@ gripe(ps)
 
 	pb->size.bytes = ftell(append_file);
 	/* next time body may stay in core */
-	mailgets      = NULL;
+	mail_parameters(ps->mail_stream, SET_GETS, (void *)NULL);
 	append_file   = NULL;
 	mail_gc(ps->mail_stream, GC_TEXTS);
 	so_release((STORE_S *)pb->contents.binary);
@@ -1624,18 +1672,13 @@ gripe(ps)
 
 	pb->size.bytes += strlen(p);
 #endif
-	break;
-
-      case 'x' :
+    }
+    else if(ch == 'x'){
 	q_status_message(0, 0, 3, "Bug report cancelled.");
 	goto bomb;
-
-      case 'n':
-      default :
-	break;
     }
 
-    pine_send(outgoing,&body,"COMPOSE BUG REPORT",NULL,&fake_reply,NULL,NULL);
+    pine_send(outgoing,&body,"COMPOSE BUG REPORT",NULL,&fake_reply,NULL,NULL,0);
     ps->mangled_screen = 1;
 
   bomb:
@@ -1676,10 +1719,13 @@ get_body_part_text(stream, body, msg_no, part_no, pc, prefix, hmm)
     char       *prefix;
     char       *hmm; 
 {
-    int            i;
-    filter_t	   filters[3];
-    long	   len;
-    char	  *err;
+    int	      i;
+    filter_t  filters[3];
+    long      len;
+    char     *err;
+#ifdef	DOS
+    char     *tmpfile_name = NULL;
+#endif
 
     /* if null body, we must be talking to a non-IMAP2bis server.
      * No MIME parsing provided, so we just grab the message text...
@@ -1697,11 +1743,21 @@ get_body_part_text(stream, body, msg_no, part_no, pc, prefix, hmm)
 #ifdef	DOS
 	if(mc->rfc822_size > MAX_MSG_INCORE
 	  || (ps_global->context_current->type & FTYPE_BBOARD)){
-	    src = FileStar;
-	    mailgets = dos_gets;	/* write fetched text to disk */
+	    src = FileStar;		/* write fetched text to disk */
+	    if(!(tmpfile_name = temp_nam(NULL, "pt"))
+	       || !(append_file = fopen(tmpfile_name, "w+b"))){
+		if(tmpfile_name)
+		  fs_give((void **)&tmpfile_name);
+
+		q_status_message1(0,2,2,"Can't build tmpfile: %s",
+				  error_description(errno));
+		return(rv);
+	    }
+
+	    mail_parameters(stream, SET_GETS, (void *)dos_gets);
 	}
-	else
-	  mailgets = NULL;		/* message stays in core */
+	else				/* message stays in core */
+	  mail_parameters(stream, SET_GETS, (void *)NULL);
 #endif
 
 	if(text = mail_fetchtext(stream, msg_no)){
@@ -1726,14 +1782,17 @@ get_body_part_text(stream, body, msg_no, part_no, pc, prefix, hmm)
 	}
 
 #ifdef	DOS
-	/* clean up dos_gets returned file pointer!! */
-	if(src == FileStar && append_file){
+	/* clean up tmp file created for dos_gets ?  If so, trash
+	 * cached knowledge, and make sure next fetch stays in core
+	 */
+	if(src == FileStar){
 	    fclose(append_file);
 	    append_file = NULL;
+	    unlink(tmpfile_name);
+	    fs_give((void **)&tmpfile_name);
+	    mail_parameters(stream, SET_GETS, (void *)NULL);
 	    mail_gc(stream, GC_TEXTS);
 	}
-
-	mailgets = NULL;		/* message stays in core */
 #endif
 	return(rv == 0);
     }
@@ -1862,7 +1921,8 @@ fetch_contents(stream, msgno, root, body)
 	    }
 #else
 	    if(body->contents.binary){
-		mailgets    =dos_gets;	/* fetched text to disk */
+		/* fetch text to disk */
+		mail_parameters(stream, SET_GETS, (void *)dos_gets);
 		append_file =(FILE *)so_text((STORE_S *)body->contents.binary);
 
 		if(mail_fetchbody(stream, msgno, pnum, &body->size.bytes)){
@@ -1873,7 +1933,7 @@ fetch_contents(stream, msgno, root, body)
 		  q_status_message1(0,1,2,"\007Error fetching part %s",pnum);
 
 		/* next time body may stay in core */
-		mailgets    = NULL;
+		mail_parameters(stream, SET_GETS, (void *)NULL);
 		append_file = NULL;
 		mail_gc(stream, GC_TEXTS);
 	    }
@@ -1897,7 +1957,8 @@ fetch_contents(stream, msgno, root, body)
 	}
 #else
 	if(body->contents.binary){
-	    mailgets    = dos_gets;	/* write fetched text to disk */
+	    /* write fetched text to disk */
+	    mail_parameters(stream, SET_GETS, (void *)dos_gets);
 	    append_file = (FILE *)so_text((STORE_S *)body->contents.binary);
 	    if(mail_fetchbody(stream, msgno, pnum, &body->size.bytes)){
 		so_release((STORE_S *)body->contents.binary);
@@ -1907,7 +1968,7 @@ fetch_contents(stream, msgno, root, body)
 	      q_status_message1(0,1,2,"\007Error fetching part %s",pnum);
 
 	    /* next time body may stay in core */
-	    mailgets    = NULL;
+	    mail_parameters(stream, SET_GETS, (void *)NULL);
 	    append_file = NULL;
 	    mail_gc(stream, GC_TEXTS);
 	}
