@@ -1,5 +1,5 @@
 #if !defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: screen.c,v 4.49 1994/08/23 23:56:27 mikes Exp $";
+static char rcsid[] = "$Id: screen.c,v 4.52 1994/10/06 22:32:43 hubert Exp $";
 #endif
 /*----------------------------------------------------------------------
 
@@ -444,9 +444,20 @@ redraw_keymenu()
 #define	MS_NEW			(0x02)
 #define	MS_ANS			(0x04)
 
+#define	STATUS_BITS(X)	(!(X) ? 0 \
+			   : (X)->deleted ? MS_DEL \
+			     : (X)->answered ? MS_ANS \
+			       : (!(X)->seen && as.stream \
+				  && (!IS_NEWS(as.stream)    \
+				      || ((X)->recent		\
+					  && F_ON(F_FAKE_NEW_IN_NEWS, \
+						  ps_global)))) ? MS_NEW : 0)
+
 #define	BAR_STATUS(X)	(((X) & MS_DEL) ? "DEL"   \
 			 : ((X) & MS_ANS) ? "ANS"   \
-		           : (as.stream && !IS_NEWS(as.stream)  \
+		           : (as.stream		      \
+			      && (!IS_NEWS(as.stream)   \
+				  || F_ON(F_FAKE_NEW_IN_NEWS, ps_global)) \
 			      && ((X) & MS_NEW)) ? "NEW" : "   ")
 
 
@@ -484,29 +495,28 @@ static struct titlebar_state {
   add a few more flags
   ----------------------------------------------------------------------*/
 char *
-status_string(stream, message_cache)
+status_string(stream, mc)
      MAILSTREAM   *stream;
-     MESSAGECACHE *message_cache;
+     MESSAGECACHE *mc;
 {
-     static char string[3];
+     static char string[2] = {'\0', '\0'};
 
-     string[1] = '\0';
-
-     if(ps_global->nr_mode) {
+     if(!mc || ps_global->nr_mode) {
          string[0] = ' ';
          return(string);
      } 
 
-     string[0] = 'N';
+     string[0] = (!stream || !IS_NEWS(stream)
+		  || (mc->recent && F_ON(F_FAKE_NEW_IN_NEWS, ps_global)))
+		   ? 'N' : ' ';
 
-     if(message_cache == NULL || message_cache->seen
-	|| (stream && IS_NEWS(stream)))
+     if(mc->seen)
        string[0] = ' ';
 
-     if(message_cache->answered)
+     if(mc->answered)
        string[0] = 'A';
 
-     if(message_cache->deleted)
+     if(mc->deleted)
        string[0] = 'D';
 
      return(string);
@@ -615,7 +625,7 @@ set_titlebar(title, stream, cntxt, folder, msgmap, display_on_screen, style,
     as.stream = stream;
     as.stream_status = (!as.stream || (as.stream == ps_global->mail_stream
 				       && ps_global->dead_stream))
-			 ? Closed : as.stream->readonly ? ReadOnly : Normal;
+			 ? Closed : as.stream->rdonly ? ReadOnly : Normal;
 
     if(as.folder_name)
       fs_give((void **)&as.folder_name);
@@ -661,10 +671,7 @@ set_titlebar(title, stream, cntxt, folder, msgmap, display_on_screen, style,
 
 	as.current_message = cpystr(message_string(msgmap, tmp_20k_buf));
         as.total_messages  = mn_get_total(msgmap);
-        as.msg_state = !mc ? 0
-			   : mc->deleted ? MS_DEL
-			     : mc->answered ? MS_ANS
-			       : !mc->seen ? MS_NEW : 0;
+        as.msg_state = STATUS_BITS(mc);
 	break;
 
       case TextPercent:
@@ -677,10 +684,7 @@ set_titlebar(title, stream, cntxt, folder, msgmap, display_on_screen, style,
 
 	as.current_message = cpystr(message_string(msgmap, tmp_20k_buf));
         as.total_messages  = mn_get_total(msgmap);
-        as.msg_state = !mc ? 0
-			   : mc->deleted ? MS_DEL
-			     : mc->answered ? MS_ANS
-			       : !mc->seen ? MS_NEW : 0;
+        as.msg_state = STATUS_BITS(mc);
 	break; 
 
       case PageNumber:
@@ -1092,17 +1096,18 @@ update_titlebar_status(mc)
 	if(as.msg_state & MS_ANS)
 	  return(0);
     }
-    else if(!mc->seen){			/* then seen */
-	if(as.msg_state & MS_NEW)
+    else if(!mc->seen && as.stream
+	    && (!IS_NEWS(as.stream)
+		|| (mc->recent && F_ON(F_FAKE_NEW_IN_NEWS, ps_global)))){
+	if(as.msg_state & MS_NEW)	/* then seen */
 	  return(0);
     }
+    else if(IS_NEWS(as.stream) && F_ON(F_FAKE_NEW_IN_NEWS, ps_global) ){
+    }
     else if(as.msg_state == 0)		/* nothing to change... */
       return(0);
 
-    as.msg_state = (mc->deleted) ? MS_DEL 
-				 : mc->answered ? MS_ANS
-				   : (!mc->seen) ? MS_NEW : 0;
-
+    as.msg_state = STATUS_BITS(mc);
     StartInverse();
     PutLine0(0, as.del_column, BAR_STATUS(as.msg_state));
     EndInverse();
