@@ -1,5 +1,5 @@
 #if !defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: signals.c,v 4.26 1994/07/26 22:03:36 mikes Exp $";
+static char rcsid[] = "$Id: signals.c,v 4.31 1994/10/11 00:27:33 mikes Exp $";
 #endif
 /*----------------------------------------------------------------------
 
@@ -65,6 +65,11 @@ static char rcsid[] = "$Id: signals.c,v 4.26 1994/07/26 22:03:36 mikes Exp $";
 
 #include "headers.h"
 
+/*
+ * call used by TERM and HUP handlers to quickly close streams
+ */
+void  fast_clean_up();
+
 
               /* SigType is defined in os.h and is either int or void */
 static SigType auger_in_signal(), winch_signal(), usr2_signal();
@@ -137,7 +142,8 @@ init_signals()
     Return all signal handling back to normal
   ----------------------------------------------------------------------*/
 void
-end_signals()
+end_signals(blockem)
+    int blockem;
 {
 #ifdef	_WINDOWS
     /* NO OP */
@@ -145,55 +151,92 @@ end_signals()
 #ifdef	DOS
     interrupt_ok();
 #else
-    dprint(5, (debugfile, "end_signals\n"));
-    if((int)signal(SIGILL,  SIG_DFL) < 0) {
+    dprint(5, (debugfile, "end_signals(%d)\n", blockem));
+    if((int)signal(SIGILL,  blockem ? SIG_IGN : SIG_DFL) < 0) {
         fprintf(stderr, "Error resetting signals: %s\n",
                 error_description(errno));
         exit(-1);
     }
-    signal(SIGTRAP, SIG_DFL);
-    signal(SIGIOT,  SIG_DFL);
-    signal(SIGEMT,  SIG_DFL);
-    signal(SIGFPE,  SIG_DFL);
-    signal(SIGBUS,  SIG_DFL);
-    signal(SIGSEGV, SIG_DFL);
-    signal(SIGSYS,  SIG_DFL);
+
+    signal(SIGTRAP, blockem ? SIG_IGN : SIG_DFL);
+    signal(SIGIOT,  blockem ? SIG_IGN : SIG_DFL);
+    signal(SIGEMT,  blockem ? SIG_IGN : SIG_DFL);
+    signal(SIGFPE,  blockem ? SIG_IGN : SIG_DFL);
+    signal(SIGBUS,  blockem ? SIG_IGN : SIG_DFL);
+    signal(SIGSEGV, blockem ? SIG_IGN : SIG_DFL);
+    signal(SIGSYS,  blockem ? SIG_IGN : SIG_DFL);
 #ifdef RESIZING
-    signal(SIGWINCH, SIG_DFL);
+    signal(SIGWINCH, blockem ? SIG_IGN : SIG_DFL);
 #endif
-    signal(SIGQUIT, SIG_DFL);
+    signal(SIGQUIT, blockem ? SIG_IGN : SIG_DFL);
 #ifdef SIGTSTP
-    signal(SIGTSTP, SIG_DFL);
+    signal(SIGTSTP, blockem ? SIG_IGN : SIG_DFL);
 #endif /* SIGTSTP */
-    signal(SIGHUP, SIG_DFL);
-    signal(SIGTERM, SIG_DFL);
+    signal(SIGHUP,  blockem ? SIG_IGN : SIG_DFL);
+    signal(SIGTERM, blockem ? SIG_IGN : SIG_DFL);
 #endif	/* !DOS */
 #endif	/* !_WINDOWS */
 }
 
 
-
-
 /*----------------------------------------------------------------------
-     Handle signals caused by aborts -- SIGSEGV, SIGILL
+     Handle signals caused by aborts -- SIGSEGV, SIGILL, etc
+
 Call panic which cleans up tty modes and then core dumps
   ----------------------------------------------------------------------*/
 static SigType
 auger_in_signal()
 {
+    end_signals(1);			/* don't catch any more signals */
     dprint(5, (debugfile, "auger_in_signal()\n"));
-    panic("Received abort signal");
+    panic("Received abort signal");	/* clean up and get out */
     /* No return from panic */
 }
 
 
-
-
 /*----------------------------------------------------------------------
-      handle cleaning up mail streams and tty modes...
+      handle hang up signal -- SIGHUP
 
 Not much to do. Rely on periodic mail file check pointing.
   ----------------------------------------------------------------------*/
+SigType
+hup_signal()
+{
+#ifndef	DOS
+    end_signals(1);			/* don't catch any more signals */
+    dprint(1, (debugfile, "\n\n** Received SIGHUP **\n\n\n\n"));
+    fast_clean_up();
+    printf("\n\nPine finished. Received hang up signal\n\n");
+#endif	/* !DOS */
+    exit(0);
+}
+
+
+/*----------------------------------------------------------------------
+      handle terminate signal -- SIGTERM
+
+Not much to do. Rely on periodic mail file check pointing.
+  ----------------------------------------------------------------------*/
+SigType
+term_signal()
+{
+#ifndef	DOS
+    end_signals(1);			/* don't catch any more signals */
+    dprint(1, (debugfile, "\n\n** Received SIGTERM **\n\n\n\n"));
+    fast_clean_up();
+    printf("\n\nPine finished. Received terminate signal\n\n");
+#endif	/* !DOS */
+    exit(0);
+}
+
+
+/*----------------------------------------------------------------------
+     Handle cleaning up mail streams and tty modes...
+Not much to do. Rely on periodic mail file check pointing.  Don't try
+cleaning up screen or flushing output since stdout is likely already
+gone.  To be safe, though, we'll at least restore the original tty mode.
+  ----------------------------------------------------------------------*/
+void
 fast_clean_up()
 {
 #ifndef	DOS
@@ -208,53 +251,16 @@ fast_clean_up()
        !ps_global->mail_stream->lock)
       mail_close(ps_global->mail_stream);
 
-    end_screen();
-    end_keyboard(F_ON(F_USE_FK,ps_global));
-    end_tty_driver(ps_global);
+    Raw(0);
 
 #ifdef DEBUG
-    fclose(debugfile);
+    if(debugfile)
+      fclose(debugfile);
 #endif    
 #endif	/* !DOS */
 }
 
 
-/*----------------------------------------------------------------------
-      handle hang up signal -- SIGHUP
-
-Not much to do. Rely on periodic mail file check pointing.
-  ----------------------------------------------------------------------*/
-SigType
-hup_signal()
-{
-#ifndef	DOS
-    signal(SIGHUP, SIG_IGN);		/* ignore any more HUPs */
-    dprint(1, (debugfile, "\n\n** Received SIGHUP **\n\n\n\n"));
-    fast_clean_up();
-    printf("\n\nPine finished. Received hang up signal\n\n");
-#endif	/* !DOS */
-    exit(0);
-}
-
-
-/*----------------------------------------------------------------------
-      handle hang up signal -- SIGTERM
-
-Not much to do. Rely on periodic mail file check pointing.
-  ----------------------------------------------------------------------*/
-SigType
-term_signal()
-{
-#ifndef	DOS
-    signal(SIGTERM, SIG_IGN);		/* ignore any more TERMs */
-    dprint(1, (debugfile, "\n\n** Received SIGTERM **\n\n\n\n"));
-    fast_clean_up();
-    printf("\n\nPine finished. Received terminate signal\n\n");
-#endif	/* !DOS */
-    exit(0);
-}
-
-
 #ifndef	DOS
 /*----------------------------------------------------------------------
       handle hang up signal -- SIGUSR2
@@ -270,9 +276,10 @@ usr2_signal()
     check_point(DoItNow);
     if(ps_global->inbox_stream != NULL && !ps_global->inbox_stream->lock
        && (c = *ps_global->inbox_stream->mailbox) != '{' && c != '*'){
-	ps_global->inbox_stream->readonly = 1; 
+	ps_global->inbox_stream->rdonly = 1; 
 	mail_ping(ps_global->inbox_stream);
-	q_status_message(0,2,2,"Another pine access to Inbox.  Session now Read-Only.");
+	q_status_message(0, 2, 2,
+		   "Another Pine is accessing Inbox.  Session now Read-Only.");
 	dprint(1, (debugfile, "** INBOX went read-only **\n\n"));
     }
 
@@ -280,9 +287,10 @@ usr2_signal()
        ps_global->mail_stream != ps_global->inbox_stream &&
        !ps_global->mail_stream->lock &&
        (c = *ps_global->mail_stream->mailbox) != '{' && c != '*'){
-	ps_global->mail_stream->readonly = 1; 
+	ps_global->mail_stream->rdonly = 1; 
 	mail_ping(ps_global->mail_stream);
-	q_status_message(0,2,2,"Another pine access to folder.  Session now Read-Only.");
+	q_status_message(0, 2, 2,
+		  "Another Pine is accessing folder.  Session now Read-Only.");
 	dprint(1, (debugfile, "** secondary folder went read-only **\n\n"));
     }
 }
