@@ -1,5 +1,5 @@
 #if !defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: status.c,v 4.27 1994/08/23 06:49:55 mikes Exp $";
+static char rcsid[] = "$Id: status.c,v 4.29 1994/10/03 22:43:31 mikes Exp $";
 #endif
 /*----------------------------------------------------------------------
 
@@ -86,7 +86,7 @@ will not let messages fly past the usr can't see.
   ----------------------------------------------------------------------*/
 struct message{
     char text[MAX_SCREEN_COLS+1];
-    long min_display_time, max_display_time;
+    int  min_display_time, max_display_time;
 };
 
 #define QUEUE_LENGTH 6
@@ -98,7 +98,7 @@ static int             message_is_displayed = 0,
                        queue_length = 0,
                        needs_clearing = 0; /* Flag set by want_to()
                                               and optionally_enter() */
-static long            displayed_time;
+static time_t          displayed_time;
 
 
 /*----------------------------------------------------------------------
@@ -289,8 +289,9 @@ void
 pause_for_current_message()
 {
     if(message_is_displayed){
-	long w = (displayed_time+currently_displayed.min_display_time)-time(0);
-	if(w > 0L)
+	int w = (int)(displayed_time - time(0))
+					+ currently_displayed.min_display_time;
+	if(w > 0)
 	  sleep(w);
 
 	message_is_displayed = 0;
@@ -312,7 +313,7 @@ messages_queued(dtime)
     long *dtime;
 {
     if(dtime && message_is_displayed)
-      *dtime = max(currently_displayed.min_display_time, 1L);
+      *dtime = (long)max(currently_displayed.min_display_time, 1);
 
     return((ps_global->in_init_seq || queue_length < 1) ? 0 : queue_length);
 }
@@ -353,31 +354,31 @@ int
 display_message(command)
      int command;
 {
-    long now, diff, rv = 0;
+    int    rv = 0;
 
     if(ps_global == NULL  || ps_global->ttyo == NULL ||
        ps_global->ttyo->screen_rows <= 1 || ps_global->in_init_seq)
       return(0);
 
-    now = time(0);
-
     if(message_is_displayed) {
 	/* message is showing */
 	if(command == ctrl('L')) {
 	    output_message(currently_displayed.text);
 	}
 	else {
-	    now = time(0);
+	    time_t now;
+	    int    diff;
 	    /* must be displayed for at least 2 seconds */
-	    diff = (displayed_time +
-                    (command == NO_OP_COMMAND || command == NO_OP_IDLE ?
-                       currently_displayed.max_display_time : 
-                         currently_displayed.min_display_time)
-                    ) - now;
-            dprint(9, (debugfile, "diff: %d, displayed_time: %d, now: %d\n",
+	    now  = time(0);
+	    diff = (int)(displayed_time - now)
+			+ ((command == NO_OP_COMMAND || command == NO_OP_IDLE)
+			    ? currently_displayed.max_display_time
+			    : currently_displayed.min_display_time);
+            dprint(9, (debugfile,
+		       "STATUS: diff:%d, displayed: %ld, now: %ld\n",
 		       diff, displayed_time, now));
             if(diff > 0)
-              return((int) diff);
+              return(diff);
 
 	    if(queue_length <= 0) {
     	        if(command != NO_OP_COMMAND && command != NO_OP_IDLE) {
@@ -406,14 +407,15 @@ display_message(command)
 	    message_is_displayed = 1;
 	    displayed_time       = time(0);
 	} while(queue_length > 0
-		&& !(rv = currently_displayed.min_display_time));
+		&& !(rv = (int)currently_displayed.min_display_time));
     }
 
     needs_clearing = 0;			/* always cleared or overwritten */
     dprint(9, (debugfile,
-               "STATUS DISPE command:%d,  disp:%d,  length:%d,  Time:%s",
-	       command, message_is_displayed,
-	       queue_length, ctime(&now)));
+               "STATUS cmd:%d, disp:%d, length:%d, max:%d, min%d",
+	       command, message_is_displayed, queue_length,
+	       currently_displayed.max_display_time,
+	       currently_displayed.min_display_time));
     fflush(stdout);
     return(rv);
 }
@@ -531,7 +533,26 @@ want_to(question, dflt, on_ctrl_C, help, display_help, flush)
     q2 = fs_get(strlen(question) + 6);
     sprintf(q2, "%.*s? ", ps_global->ttyo->screen_cols - 6, question);
     rv = radio_buttons(q2, ps_global->ttyo->screen_rows > 4 ? -3 : -1, 0,
-                       yorn, dflt, on_ctrl_C, flush, help);
+                       yorn, dflt, on_ctrl_C, flush, help, 0);
+    fs_give((void **)&q2);
+
+    return(rv);
+}
+
+
+int
+one_try_want_to(question, dflt, on_ctrl_C, help, display_help, flush)
+char      *question;
+HelpType   help;
+int    dflt, on_ctrl_C, display_help, flush;
+{
+    char     *q2;
+    int	      rv;
+
+    q2 = fs_get(strlen(question) + 6);
+    sprintf(q2, "%.*s? ", ps_global->ttyo->screen_cols - 6, question);
+    rv = radio_buttons(q2, ps_global->ttyo->screen_rows > 4 ? -3 : -1, 0,
+                       yorn, dflt, on_ctrl_C, flush, help, 1);
     fs_give((void **)&q2);
 
     return(rv);
@@ -542,30 +563,39 @@ want_to(question, dflt, on_ctrl_C, help, display_help, flush)
 /*----------------------------------------------------------------------
     Prompt user for a choice among alternatives
 
-Args --  prompt: The prompt for the question/selection
-         line:   The line to prompt on, if negative then relative to bottom
-         column:  Column number to prompt on
-         buttons: The letter the user types to push the buttons
-         labels:  String labels on the buttons. Displayed when pushed
-         deefault: The selection when the <CR> is pressed (should probably
-		   be one of the chars in buttons)
+Args --  prompt:    The prompt for the question/selection
+         line:      The line to prompt on, if negative then relative to bottom
+         column:    Column number to prompt on
+         esc_list:  ESC_KEY_S list of keys
+         deefault:  The selection when the <CR> is pressed (should probably
+		      be one of the chars in esc_list)
          on_ctrl_C: The selection when ^C is pressed
+	 flush:     Discard any pending input chars.
          help_text: Text to be displayed on bottom two lines
+	 one_try:   Only give one chance to answer.  Returns on_ctrl_C value
+		      if not answered acceptably on first try.
 
 Result -- Returns the letter pressed. Will be one of the characters in the
-          buttons argument or one of the two deefaults.
+          esc_list argument or one of the two deefaults.
 
 This will pause for any new status message to be seen and then prompt the user.
 The prompt will be truncated to fit on the screen. Redraw and resize are
 handled along with ^Z suspension. Typing ^G will toggle the help text on and
-off. Character types that are not buttons wil result in a beep.
+off. Character types that are not buttons will result in a beep (unless one_try
+is set).
   ----*/
 int
-radio_buttons(prompt,line,column,esc_list,deefault,on_ctrl_C,flush,help_text)
-     char	*prompt;
-     ESCKEY_S   *esc_list;
-     HelpType	 help_text;
-     int	 line, column, deefault, on_ctrl_C, flush;
+radio_buttons(prompt, line, column, esc_list, deefault, on_ctrl_C, flush,
+		help_text, one_try)
+char	 *prompt;
+int       line;
+int       column;
+ESCKEY_S *esc_list;
+int       deefault;
+int       on_ctrl_C;
+int       flush;
+HelpType  help_text;
+int       one_try;
 {
     register int     ch, real_line;
     char            *q, *ds = NULL;
@@ -669,12 +699,16 @@ radio_buttons(prompt,line,column,esc_list,deefault,on_ctrl_C,flush,help_text)
 	       || ((ch >= PF1 && ch <= PF12)
 		   && (ch = fkey_table[ch - PF1]) == NO_OP_COMMAND))){
 	    /*
-	     * The funcky test above does two things.  First, it maps
+	     * The funky test above does two things.  First, it maps
 	     * esc_list character commands to function keys, *and* prevents
 	     * character commands from input while in function key mode.
 	     * NOTE: this breaks if we ever need more than the first
 	     * twelve function keys...
 	     */
+	    if(one_try){
+		ch = on_ctrl_C;
+	        goto out_of_loop;
+	    }
 	    Writechar('\007', 0);
 	    continue;
 	}
@@ -689,6 +723,10 @@ radio_buttons(prompt,line,column,esc_list,deefault,on_ctrl_C,flush,help_text)
 		  goto out_of_loop;
 	      }
 
+	    if(one_try){
+		ch = on_ctrl_C;
+	        goto out_of_loop;
+	    }
 	    Writechar('\007', 0);
 	    break;
 
@@ -698,7 +736,7 @@ radio_buttons(prompt,line,column,esc_list,deefault,on_ctrl_C,flush,help_text)
             goto out_of_loop;
 
           case ctrl('C'):
-	    if(on_ctrl_C){
+	    if(on_ctrl_C || one_try){
 		ch = on_ctrl_C;
 		goto out_of_loop;
 	    }
