@@ -1,5 +1,5 @@
 #if !defined(lint) && !defined(DOS)
-static char rcsid[] = "$Id: ttyout.c,v 4.26 1994/08/16 03:18:27 mikes Exp $";
+static char rcsid[] = "$Id: ttyout.c,v 4.28 1994/09/21 22:29:18 hubert Exp $";
 #endif
 /*----------------------------------------------------------------------
 
@@ -450,6 +450,7 @@ ClearScreen()
         MoveCursor(0,0);
     } else {
 	tputs(_clearscreen, 1, outchar);
+        moveabsolute(0, 0);  /* some clearscreens don't move correctly */
     }
 }
 
@@ -1514,12 +1515,13 @@ struct ttyo *ttyo;
 #ifdef _WINDOWS
     char	fontName[LF_FACESIZE+1];
     char	fontSize[12];
+    char	fontStyle[64];
     char	windowPosition[32];
     int		newRows, newCols;
     
 	    
     /* Get the new window parameters and update the 'pinerc' variables. */
-    mswin_getwindow (fontName, fontSize, windowPosition);
+    mswin_getwindow (fontName, fontSize, fontStyle, windowPosition);
     if(!ps_global->VAR_FONT_NAME
        || strucmp(ps_global->VAR_FONT_NAME, fontName))
       set_variable(V_FONT_NAME, fontName, 0);
@@ -1528,6 +1530,10 @@ struct ttyo *ttyo;
        || strucmp(ps_global->VAR_FONT_SIZE, fontSize))
       set_variable(V_FONT_SIZE, fontSize, 0);
 
+    if(!ps_global->VAR_FONT_STYLE
+       || strucmp(ps_global->VAR_FONT_STYLE, fontStyle))
+      set_variable(V_FONT_STYLE, fontStyle, 0);
+
     if(!ps_global->VAR_WINDOW_POSITION
        || strucmp(ps_global->VAR_WINDOW_POSITION, windowPosition))
       set_variable(V_WINDOW_POSITION, windowPosition, 0);
